@@ -1,9 +1,10 @@
 //! Module defining the AST (abstract syntax tree).
 
+use crate::calc_fn_hash;
+use crate::dynamic::Union;
 use crate::fn_native::shared_make_mut;
 use crate::module::NamespaceRef;
 use crate::token::Token;
-use crate::utils::calc_fn_hash;
 use crate::{
     Dynamic, FnNamespace, Identifier, ImmutableString, Module, Position, Shared, StaticVec, INT,
 };
@@ -90,7 +91,7 @@ impl fmt::Display for ScriptFnDef {
             self.params
                 .iter()
                 .map(|s| s.as_str())
-                .collect::<Vec<_>>()
+                .collect::<StaticVec<_>>()
                 .join(", ")
         )
     }
@@ -138,7 +139,11 @@ impl fmt::Display for ScriptFnMetadata<'_> {
                 FnAccess::Private => "private ",
             },
             self.name,
-            self.params.iter().cloned().collect::<Vec<_>>().join(", ")
+            self.params
+                .iter()
+                .cloned()
+                .collect::<StaticVec<_>>()
+                .join(", ")
         )
     }
 }
@@ -211,14 +216,14 @@ impl Default for AST {
 impl AST {
     /// Create a new [`AST`].
     #[inline(always)]
+    #[must_use]
     pub fn new(
         statements: impl IntoIterator<Item = Stmt>,
         functions: impl Into<Shared<Module>>,
     ) -> Self {
-        let statements: StaticVec<_> = statements.into_iter().collect();
         Self {
             source: None,
-            body: StmtBlock::new(statements, Position::NONE),
+            body: StmtBlock::new(statements.into_iter().collect(), Position::NONE),
             functions: functions.into(),
             #[cfg(not(feature = "no_module"))]
             resolver: None,
@@ -226,15 +231,15 @@ impl AST {
     }
     /// Create a new [`AST`] with a source name.
     #[inline(always)]
+    #[must_use]
     pub fn new_with_source(
         statements: impl IntoIterator<Item = Stmt>,
         functions: impl Into<Shared<Module>>,
         source: impl Into<Identifier>,
     ) -> Self {
-        let statements: StaticVec<_> = statements.into_iter().collect();
         Self {
             source: Some(source.into()),
-            body: StmtBlock::new(statements, Position::NONE),
+            body: StmtBlock::new(statements.into_iter().collect(), Position::NONE),
             functions: functions.into(),
             #[cfg(not(feature = "no_module"))]
             resolver: None,
@@ -242,22 +247,24 @@ impl AST {
     }
     /// Get the source, if any.
     #[inline(always)]
+    #[must_use]
     pub fn source(&self) -> Option<&str> {
         self.source.as_ref().map(|s| s.as_str())
     }
-    /// Clone the source, if any.
+    /// Get a reference to the source.
     #[inline(always)]
-    pub(crate) fn clone_source(&self) -> Option<Identifier> {
-        self.source.clone()
+    #[must_use]
+    pub(crate) fn source_raw(&self) -> Option<&Identifier> {
+        self.source.as_ref()
     }
     /// Set the source.
     #[inline(always)]
     pub fn set_source(&mut self, source: impl Into<Identifier>) -> &mut Self {
-        let source = Some(source.into());
+        let source = source.into();
         Shared::get_mut(&mut self.functions)
             .as_mut()
             .map(|m| m.set_id(source.clone()));
-        self.source = source;
+        self.source = Some(source);
         self
     }
     /// Clear the source.
@@ -269,6 +276,7 @@ impl AST {
     /// Get the statements.
     #[cfg(not(feature = "internals"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn statements(&self) -> &[Stmt] {
         &self.body.0
     }
@@ -277,12 +285,14 @@ impl AST {
     #[cfg(feature = "internals")]
     #[deprecated = "this method is volatile and may change"]
     #[inline(always)]
+    #[must_use]
     pub fn statements(&self) -> &[Stmt] {
         &self.body.0
     }
     /// Get a mutable reference to the statements.
     #[cfg(not(feature = "no_optimize"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn statements_mut(&mut self) -> &mut StaticVec<Stmt> {
         &mut self.body.0
     }
@@ -291,6 +301,7 @@ impl AST {
     #[cfg(not(feature = "no_module"))]
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn shared_lib(&self) -> Shared<Module> {
         self.functions.clone()
     }
@@ -303,12 +314,14 @@ impl AST {
     #[cfg(not(feature = "no_module"))]
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn shared_lib(&self) -> Shared<Module> {
         self.functions.clone()
     }
     /// Get the internal [`Module`] containing all script-defined functions.
     #[cfg(not(feature = "internals"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn lib(&self) -> &Module {
         &self.functions
     }
@@ -319,6 +332,7 @@ impl AST {
     #[cfg(feature = "internals")]
     #[deprecated = "this method is volatile and may change"]
     #[inline(always)]
+    #[must_use]
     pub fn lib(&self) -> &Module {
         &self.functions
     }
@@ -326,6 +340,7 @@ impl AST {
     #[cfg(not(feature = "no_module"))]
     #[cfg(not(feature = "internals"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn resolver(
         &self,
     ) -> Option<Shared<crate::module::resolvers::StaticModuleResolver>> {
@@ -338,6 +353,7 @@ impl AST {
     #[cfg(not(feature = "no_module"))]
     #[cfg(feature = "internals")]
     #[inline(always)]
+    #[must_use]
     pub fn resolver(&self) -> Option<Shared<crate::module::resolvers::StaticModuleResolver>> {
         self.resolver.clone()
     }
@@ -359,6 +375,7 @@ impl AST {
     /// This operation is cheap because functions are shared.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn clone_functions_only(&self) -> Self {
         self.clone_functions_only_filtered(|_, _, _, _, _| true)
     }
@@ -370,6 +387,7 @@ impl AST {
     /// This operation is cheap because functions are shared.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn clone_functions_only_filtered(
         &self,
         filter: impl Fn(FnNamespace, FnAccess, bool, &str, usize) -> bool,
@@ -387,6 +405,7 @@ impl AST {
     /// Clone the [`AST`]'s script statements into a new [`AST`].
     /// No functions are cloned.
     #[inline(always)]
+    #[must_use]
     pub fn clone_statements_only(&self) -> Self {
         Self {
             source: self.source.clone(),
@@ -417,15 +436,15 @@ impl AST {
     ///
     /// let engine = Engine::new();
     ///
-    /// let ast1 = engine.compile(r#"
-    ///                 fn foo(x) { 42 + x }
-    ///                 foo(1)
-    ///             "#)?;
+    /// let ast1 = engine.compile("
+    ///     fn foo(x) { 42 + x }
+    ///     foo(1)
+    /// ")?;
     ///
     /// let ast2 = engine.compile(r#"
-    ///                 fn foo(n) { `hello${n}` }
-    ///                 foo("!")
-    ///             "#)?;
+    ///     fn foo(n) { `hello${n}` }
+    ///     foo("!")
+    /// "#)?;
     ///
     /// let ast = ast1.merge(&ast2);    // Merge 'ast2' into 'ast1'
     ///
@@ -446,6 +465,7 @@ impl AST {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn merge(&self, other: &Self) -> Self {
         self.merge_filtered(other, |_, _, _, _, _| true)
     }
@@ -469,15 +489,15 @@ impl AST {
     ///
     /// let engine = Engine::new();
     ///
-    /// let mut ast1 = engine.compile(r#"
-    ///                     fn foo(x) { 42 + x }
-    ///                     foo(1)
-    ///                 "#)?;
+    /// let mut ast1 = engine.compile("
+    ///     fn foo(x) { 42 + x }
+    ///     foo(1)
+    /// ")?;
     ///
     /// let ast2 = engine.compile(r#"
-    ///                 fn foo(n) { `hello${n}` }
-    ///                 foo("!")
-    ///             "#)?;
+    ///     fn foo(n) { `hello${n}` }
+    ///     foo("!")
+    /// "#)?;
     ///
     /// ast1.combine(ast2);    // Combine 'ast2' into 'ast1'
     ///
@@ -523,16 +543,16 @@ impl AST {
     ///
     /// let engine = Engine::new();
     ///
-    /// let ast1 = engine.compile(r#"
-    ///                 fn foo(x) { 42 + x }
-    ///                 foo(1)
-    ///             "#)?;
+    /// let ast1 = engine.compile("
+    ///     fn foo(x) { 42 + x }
+    ///     foo(1)
+    /// ")?;
     ///
     /// let ast2 = engine.compile(r#"
-    ///                 fn foo(n) { `hello${n}` }
-    ///                 fn error() { 0 }
-    ///                 foo("!")
-    ///             "#)?;
+    ///     fn foo(n) { `hello${n}` }
+    ///     fn error() { 0 }
+    ///     foo("!")
+    /// "#)?;
     ///
     /// // Merge 'ast2', picking only 'error()' but not 'foo(_)', into 'ast1'
     /// let ast = ast1.merge_filtered(&ast2, |_, _, script, name, params|
@@ -554,6 +574,7 @@ impl AST {
     /// # }
     /// ```
     #[inline]
+    #[must_use]
     pub fn merge_filtered(
         &self,
         other: &Self,
@@ -606,16 +627,16 @@ impl AST {
     ///
     /// let engine = Engine::new();
     ///
-    /// let mut ast1 = engine.compile(r#"
-    ///                     fn foo(x) { 42 + x }
-    ///                     foo(1)
-    ///                 "#)?;
+    /// let mut ast1 = engine.compile("
+    ///     fn foo(x) { 42 + x }
+    ///     foo(1)
+    /// ")?;
     ///
     /// let ast2 = engine.compile(r#"
-    ///                 fn foo(n) { `hello${n}` }
-    ///                 fn error() { 0 }
-    ///                 foo("!")
-    ///             "#)?;
+    ///     fn foo(n) { `hello${n}` }
+    ///     fn error() { 0 }
+    ///     foo("!")
+    /// "#)?;
     ///
     /// // Combine 'ast2', picking only 'error()' but not 'foo(_)', into 'ast1'
     /// ast1.combine_filtered(ast2, |_, _, script, name, params|
@@ -664,9 +685,9 @@ impl AST {
     /// let engine = Engine::new();
     ///
     /// let mut ast = engine.compile(r#"
-    ///                         fn foo(n) { n + 1 }
-    ///                         fn bar() { print("hello"); }
-    ///                     "#)?;
+    ///     fn foo(n) { n + 1 }
+    ///     fn bar() { print("hello"); }
+    /// "#)?;
     ///
     /// // Remove all functions except 'foo(_)'
     /// ast.retain_functions(|_, _, name, params| name == "foo" && params == 1);
@@ -691,6 +712,7 @@ impl AST {
     #[cfg(not(feature = "no_function"))]
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn iter_fn_def(&self) -> impl Iterator<Item = &ScriptFnDef> {
         self.functions
             .iter_script_fn()
@@ -701,6 +723,7 @@ impl AST {
     /// Not available under `no_function`.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn iter_functions<'a>(&'a self) -> impl Iterator<Item = ScriptFnMetadata> + 'a {
         self.functions
             .iter_script_fn()
@@ -711,13 +734,15 @@ impl AST {
     /// Not available under `no_function`.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
-    pub fn clear_functions(&mut self) {
+    pub fn clear_functions(&mut self) -> &mut Self {
         self.functions = Default::default();
+        self
     }
     /// Clear all statements in the [`AST`], leaving only function definitions.
     #[inline(always)]
-    pub fn clear_statements(&mut self) {
+    pub fn clear_statements(&mut self) -> &mut Self {
         self.body = Default::default();
+        self
     }
     /// Recursively walk the [`AST`], including function bodies (if any).
     /// Return `false` from the callback to terminate the walk.
@@ -866,27 +891,33 @@ pub struct StmtBlock(StaticVec<Stmt>, Position);
 
 impl StmtBlock {
     /// Create a new [`StmtBlock`].
-    pub fn new(statements: impl Into<StaticVec<Stmt>>, pos: Position) -> Self {
-        let mut statements = statements.into();
+    #[must_use]
+    pub fn new(mut statements: StaticVec<Stmt>, pos: Position) -> Self {
         statements.shrink_to_fit();
         Self(statements, pos)
     }
     /// Is this statements block empty?
     #[inline(always)]
+    #[must_use]
     pub fn is_empty(&self) -> bool {
         self.0.is_empty()
     }
     /// Number of statements in this statements block.
     #[inline(always)]
+    #[must_use]
     pub fn len(&self) -> usize {
         self.0.len()
     }
     /// Get the position of this statements block.
-    pub fn position(&self) -> Position {
+    #[inline(always)]
+    #[must_use]
+    pub const fn position(&self) -> Position {
         self.1
     }
     /// Get the statements of this statements block.
-    pub fn statements(&mut self) -> &mut StaticVec<Stmt> {
+    #[inline(always)]
+    #[must_use]
+    pub fn statements_mut(&mut self) -> &mut StaticVec<Stmt> {
         &mut self.0
     }
 }
@@ -894,12 +925,14 @@ impl StmtBlock {
 impl Deref for StmtBlock {
     type Target = StaticVec<Stmt>;
 
+    #[inline(always)]
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
 impl DerefMut for StmtBlock {
+    #[inline(always)]
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.0
     }
@@ -915,6 +948,7 @@ impl fmt::Debug for StmtBlock {
 }
 
 impl From<StmtBlock> for Stmt {
+    #[inline(always)]
     fn from(block: StmtBlock) -> Self {
         let block_pos = block.position();
         Self::Block(block.0.into_boxed_slice(), block_pos)
@@ -943,14 +977,14 @@ pub enum Stmt {
     While(Expr, Box<StmtBlock>, Position),
     /// `do` `{` stmt `}` `while`|`until` expr
     Do(Box<StmtBlock>, Expr, bool, Position),
-    /// `for` id `in` expr `{` stmt `}`
-    For(Expr, Box<(Ident, StmtBlock)>, Position),
+    /// `for` `(` id `,` counter `)` `in` expr `{` stmt `}`
+    For(Expr, Box<(Ident, Option<Ident>, StmtBlock)>, Position),
     /// \[`export`\] `let` id `=` expr
     Let(Expr, Box<Ident>, bool, Position),
     /// \[`export`\] `const` id `=` expr
     Const(Expr, Box<Ident>, bool, Position),
     /// expr op`=` expr
-    Assignment(Box<(Expr, Option<OpAssignment>, Expr)>, Position),
+    Assignment(Box<(Expr, Option<OpAssignment<'static>>, Expr)>, Position),
     /// func `(` expr `,` ... `)`
     ///
     /// Note - this is a duplicate of [`Expr::FnCall`] to cover the very common pattern of a single
@@ -959,11 +993,7 @@ pub enum Stmt {
     /// `{` stmt`;` ... `}`
     Block(Box<[Stmt]>, Position),
     /// `try` `{` stmt; ... `}` `catch` `(` var `)` `{` stmt; ... `}`
-    TryCatch(
-        Box<(StmtBlock, Option<Ident>, StmtBlock)>,
-        Position,
-        Position,
-    ),
+    TryCatch(Box<(StmtBlock, Option<Ident>, StmtBlock)>, Position),
     /// [expression][Expr]
     Expr(Expr),
     /// `continue`
@@ -981,7 +1011,7 @@ pub enum Stmt {
     ///
     /// Not available under `no_module`.
     #[cfg(not(feature = "no_module"))]
-    Export(Box<[(Ident, Option<Ident>)]>, Position),
+    Export(Box<[(Ident, Ident)]>, Position),
     /// Convert a variable to shared.
     ///
     /// Not available under `no_closure`.
@@ -1000,9 +1030,7 @@ impl From<Stmt> for StmtBlock {
     #[inline(always)]
     fn from(stmt: Stmt) -> Self {
         match stmt {
-            Stmt::Block(mut block, pos) => {
-                Self(block.iter_mut().map(|v| mem::take(v)).collect(), pos)
-            }
+            Stmt::Block(mut block, pos) => Self(block.iter_mut().map(mem::take).collect(), pos),
             Stmt::Noop(pos) => Self(Default::default(), pos),
             _ => {
                 let pos = stmt.position();
@@ -1015,14 +1043,16 @@ impl From<Stmt> for StmtBlock {
 impl Stmt {
     /// Is this statement [`Noop`][Stmt::Noop]?
     #[inline(always)]
-    pub fn is_noop(&self) -> bool {
+    #[must_use]
+    pub const fn is_noop(&self) -> bool {
         match self {
             Self::Noop(_) => true,
             _ => false,
         }
     }
     /// Get the [position][Position] of this statement.
-    pub fn position(&self) -> Position {
+    #[must_use]
+    pub const fn position(&self) -> Position {
         match self {
             Self::Noop(pos)
             | Self::Continue(pos)
@@ -1038,7 +1068,7 @@ impl Stmt {
             | Self::Return(_, _, pos)
             | Self::Let(_, _, _, pos)
             | Self::Const(_, _, _, pos)
-            | Self::TryCatch(_, pos, _) => *pos,
+            | Self::TryCatch(_, pos) => *pos,
 
             Self::Expr(x) => x.position(),
 
@@ -1068,7 +1098,7 @@ impl Stmt {
             | Self::Return(_, _, pos)
             | Self::Let(_, _, _, pos)
             | Self::Const(_, _, _, pos)
-            | Self::TryCatch(_, pos, _) => *pos = new_pos,
+            | Self::TryCatch(_, pos) => *pos = new_pos,
 
             Self::Expr(x) => {
                 x.set_position(new_pos);
@@ -1086,7 +1116,8 @@ impl Stmt {
         self
     }
     /// Does this statement return a value?
-    pub fn returns_value(&self) -> bool {
+    #[must_use]
+    pub const fn returns_value(&self) -> bool {
         match self {
             Self::If(_, _, _)
             | Self::Switch(_, _, _)
@@ -1098,7 +1129,7 @@ impl Stmt {
             | Self::While(_, _, _)
             | Self::Do(_, _, _, _)
             | Self::For(_, _, _)
-            | Self::TryCatch(_, _, _) => false,
+            | Self::TryCatch(_, _) => false,
 
             Self::Let(_, _, _, _)
             | Self::Const(_, _, _, _)
@@ -1111,18 +1142,19 @@ impl Stmt {
             Self::Import(_, _, _) | Self::Export(_, _) => false,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => unreachable!("Stmt::Share should not be parsed"),
+            Self::Share(_) => false,
         }
     }
     /// Is this statement self-terminated (i.e. no need for a semicolon terminator)?
-    pub fn is_self_terminated(&self) -> bool {
+    #[must_use]
+    pub const fn is_self_terminated(&self) -> bool {
         match self {
             Self::If(_, _, _)
             | Self::Switch(_, _, _)
             | Self::While(_, _, _)
             | Self::For(_, _, _)
             | Self::Block(_, _)
-            | Self::TryCatch(_, _, _) => true,
+            | Self::TryCatch(_, _) => true,
 
             // A No-op requires a semicolon in order to know it is an empty statement!
             Self::Noop(_) => false,
@@ -1141,12 +1173,13 @@ impl Stmt {
             Self::Import(_, _, _) | Self::Export(_, _) => false,
 
             #[cfg(not(feature = "no_closure"))]
-            Self::Share(_) => unreachable!("Stmt::Share should not be parsed"),
+            Self::Share(_) => false,
         }
     }
     /// Is this statement _pure_?
     ///
     /// A pure statement has no side effects.
+    #[must_use]
     pub fn is_pure(&self) -> bool {
         match self {
             Self::Noop(_) => true,
@@ -1167,14 +1200,14 @@ impl Stmt {
             Self::While(condition, block, _) | Self::Do(block, condition, _, _) => {
                 condition.is_pure() && block.0.iter().all(Stmt::is_pure)
             }
-            Self::For(iterable, x, _) => iterable.is_pure() && (x.1).0.iter().all(Stmt::is_pure),
+            Self::For(iterable, x, _) => iterable.is_pure() && (x.2).0.iter().all(Stmt::is_pure),
             Self::Let(_, _, _, _)
             | Self::Const(_, _, _, _)
             | Self::Assignment(_, _)
             | Self::FnCall(_, _) => false,
             Self::Block(block, _) => block.iter().all(|stmt| stmt.is_pure()),
             Self::Continue(_) | Self::Break(_) | Self::Return(_, _, _) => false,
-            Self::TryCatch(x, _, _) => {
+            Self::TryCatch(x, _) => {
                 (x.0).0.iter().all(Stmt::is_pure) && (x.2).0.iter().all(Stmt::is_pure)
             }
 
@@ -1194,6 +1227,7 @@ impl Stmt {
     /// Only variable declarations (i.e. `let` and `const`) and `import`/`export` statements
     /// are internally pure.
     #[inline(always)]
+    #[must_use]
     pub fn is_internally_pure(&self) -> bool {
         match self {
             Self::Let(expr, _, _, _) | Self::Const(expr, _, _, _) => expr.is_pure(),
@@ -1212,7 +1246,8 @@ impl Stmt {
     ///
     /// All statements following this statement will essentially be dead code.
     #[inline(always)]
-    pub fn is_control_flow_break(&self) -> bool {
+    #[must_use]
+    pub const fn is_control_flow_break(&self) -> bool {
         match self {
             Self::Return(_, _, _) | Self::Break(_) | Self::Continue(_) => true,
             _ => false,
@@ -1287,7 +1322,7 @@ impl Stmt {
                 if !e.walk(path, on_node) {
                     return false;
                 }
-                for s in &(x.1).0 {
+                for s in &(x.2).0 {
                     if !s.walk(path, on_node) {
                         return false;
                     }
@@ -1315,7 +1350,7 @@ impl Stmt {
                     }
                 }
             }
-            Self::TryCatch(x, _, _) => {
+            Self::TryCatch(x, _) => {
                 for s in &(x.0).0 {
                     if !s.walk(path, on_node) {
                         return false;
@@ -1359,7 +1394,7 @@ pub struct CustomExpr {
     /// List of keywords.
     pub keywords: StaticVec<Expr>,
     /// Is the current [`Scope`][crate::Scope] modified?
-    pub scope_changed: bool,
+    pub scope_may_be_changed: bool,
     /// List of tokens actually parsed.
     pub tokens: StaticVec<Identifier>,
 }
@@ -1384,19 +1419,23 @@ pub struct BinaryExpr {
 /// # Volatile Data Structure
 ///
 /// This type is volatile and may change.
-#[derive(Debug, Clone, Eq, PartialEq, Hash)]
-pub struct OpAssignment {
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
+pub struct OpAssignment<'a> {
+    /// Hash of the op-assignment call.
     pub hash_op_assign: u64,
+    /// Hash of the underlying operator call.
     pub hash_op: u64,
-    pub op: &'static str,
+    /// Op-assignment operator.
+    pub op: &'a str,
 }
 
-impl OpAssignment {
+impl OpAssignment<'_> {
     /// Create a new [`OpAssignment`].
     ///
     /// # Panics
     ///
     /// Panics if the operator name is not an op-assignment operator.
+    #[must_use]
     pub fn new(op: Token) -> Self {
         let op_raw = op
             .map_op_assignment()
@@ -1453,12 +1492,12 @@ impl fmt::Debug for FnCallHashes {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         if let Some(script) = self.script {
             if script == self.native {
-                write!(f, "({}=={})", script, self.native)
+                fmt::Debug::fmt(&self.native, f)
             } else {
                 write!(f, "({}, {})", script, self.native)
             }
         } else {
-            write!(f, "{}", self.native)
+            write!(f, "{} (native only)", self.native)
         }
     }
 }
@@ -1466,7 +1505,8 @@ impl fmt::Debug for FnCallHashes {
 impl FnCallHashes {
     /// Create a [`FnCallHashes`] with only the native Rust hash.
     #[inline(always)]
-    pub fn from_native(hash: u64) -> Self {
+    #[must_use]
+    pub const fn from_native(hash: u64) -> Self {
         Self {
             script: None,
             native: hash,
@@ -1474,7 +1514,8 @@ impl FnCallHashes {
     }
     /// Create a [`FnCallHashes`] with both native Rust and script function hashes set to the same value.
     #[inline(always)]
-    pub fn from_script(hash: u64) -> Self {
+    #[must_use]
+    pub const fn from_script(hash: u64) -> Self {
         Self {
             script: Some(hash),
             native: hash,
@@ -1482,7 +1523,8 @@ impl FnCallHashes {
     }
     /// Create a [`FnCallHashes`] with both native Rust and script function hashes.
     #[inline(always)]
-    pub fn from_script_and_native(script: u64, native: u64) -> Self {
+    #[must_use]
+    pub const fn from_script_and_native(script: u64, native: u64) -> Self {
         Self {
             script: Some(script),
             native,
@@ -1490,19 +1532,10 @@ impl FnCallHashes {
     }
     /// Is this [`FnCallHashes`] native Rust only?
     #[inline(always)]
-    pub fn is_native_only(&self) -> bool {
+    #[must_use]
+    pub const fn is_native_only(&self) -> bool {
         self.script.is_none()
     }
-    /// Get the script function hash from this [`FnCallHashes`].
-    #[inline(always)]
-    pub fn script_hash(&self) -> Option<u64> {
-        self.script
-    }
-    /// Get the naive Rust function hash from this [`FnCallHashes`].
-    #[inline(always)]
-    pub fn native_hash(&self) -> u64 {
-        self.native
-    }
 }
 
 /// _(INTERNALS)_ A function call.
@@ -1520,7 +1553,7 @@ pub struct FnCallExpr {
     /// List of function call argument expressions.
     pub args: StaticVec<Expr>,
     /// List of function call arguments that are constants.
-    pub literal_args: smallvec::SmallVec<[(Dynamic, Position); 2]>,
+    pub constants: smallvec::SmallVec<[Dynamic; 2]>,
     /// Function name.
     pub name: Identifier,
     /// Does this function call capture the parent scope?
@@ -1530,18 +1563,15 @@ pub struct FnCallExpr {
 impl FnCallExpr {
     /// Does this function call contain a qualified namespace?
     #[inline(always)]
-    pub fn is_qualified(&self) -> bool {
+    #[must_use]
+    pub const fn is_qualified(&self) -> bool {
         self.namespace.is_some()
     }
-    /// Are there no arguments to this function call?
+    /// Convert this into a [`FnCall`][Expr::FnCall].
     #[inline(always)]
-    pub fn is_args_empty(&self) -> bool {
-        self.args.is_empty() && self.literal_args.is_empty()
-    }
-    /// Get the number of arguments to this function call.
-    #[inline(always)]
-    pub fn args_count(&self) -> usize {
-        self.args.len() + self.literal_args.len()
+    #[must_use]
+    pub fn into_fn_call_expr(self, pos: Position) -> Expr {
+        Expr::FnCall(self.into(), pos)
     }
 }
 
@@ -1647,7 +1677,9 @@ impl<F: Float> FloatWrapper<F> {
     /// Minimum floating-point number for natural display before switching to scientific notation.
     pub const MIN_NATURAL_FLOAT_FOR_DISPLAY: f32 = 0.0000000000001;
 
+    /// Create a new [`FloatWrapper`].
     #[inline(always)]
+    #[must_use]
     pub fn new(value: F) -> Self {
         Self(value)
     }
@@ -1655,7 +1687,9 @@ impl<F: Float> FloatWrapper<F> {
 
 #[cfg(not(feature = "no_float"))]
 impl FloatWrapper<FLOAT> {
+    /// Create a new [`FloatWrapper`].
     #[inline(always)]
+    #[must_use]
     pub(crate) const fn const_new(value: FLOAT) -> Self {
         Self(value)
     }
@@ -1687,7 +1721,7 @@ pub enum Expr {
     /// [String][ImmutableString] constant.
     StringConstant(ImmutableString, Position),
     /// An interpolated [string][ImmutableString].
-    InterpolatedString(Box<StaticVec<Expr>>),
+    InterpolatedString(Box<StaticVec<Expr>>, Position),
     /// [ expr, ... ]
     Array(Box<StaticVec<Expr>>, Position),
     /// #{ name:expr, ... }
@@ -1707,7 +1741,7 @@ pub enum Expr {
         Position,
         Box<(
             Option<NonZeroUsize>,
-            Option<(u64, NamespaceRef)>,
+            Option<(NamespaceRef, u64)>,
             Identifier,
         )>,
     ),
@@ -1719,6 +1753,8 @@ pub enum Expr {
             (ImmutableString, Position),
         )>,
     ),
+    /// Stack slot
+    Stack(usize, Position),
     /// { [statement][Stmt] ... }
     Stmt(Box<StmtBlock>),
     /// func `(` expr `,` ... `)`
@@ -1756,7 +1792,7 @@ impl fmt::Debug for Expr {
             Self::StringConstant(value, _) => write!(f, "{:?}", value),
             Self::Unit(_) => f.write_str("()"),
 
-            Self::InterpolatedString(x) => {
+            Self::InterpolatedString(x, _) => {
                 f.write_str("InterpolatedString")?;
                 return f.debug_list().entries(x.iter()).finish();
             }
@@ -1776,14 +1812,15 @@ impl fmt::Debug for Expr {
                     Some((_, ref namespace)) => write!(f, "{}", namespace)?,
                     _ => (),
                 }
-                write!(f, "{}", x.2)?;
+                f.write_str(&x.2)?;
                 match i.map_or_else(|| x.0, |n| NonZeroUsize::new(n.get() as usize)) {
-                    Some(n) => write!(f, ", {}", n)?,
+                    Some(n) => write!(f, " #{}", n)?,
                     _ => (),
                 }
                 f.write_str(")")
             }
             Self::Property(x) => write!(f, "Property({})", (x.2).0),
+            Self::Stack(x, _) => write!(f, "StackSlot({})", x),
             Self::Stmt(x) => {
                 f.write_str("ExprStmtBlock")?;
                 f.debug_list().entries(x.0.iter()).finish()
@@ -1794,8 +1831,8 @@ impl fmt::Debug for Expr {
                 ff.field("name", &x.name)
                     .field("hash", &x.hashes)
                     .field("args", &x.args);
-                if !x.literal_args.is_empty() {
-                    ff.field("literal_args", &x.literal_args);
+                if !x.constants.is_empty() {
+                    ff.field("constants", &x.constants);
                 }
                 if x.capture {
                     ff.field("capture", &x.capture);
@@ -1826,11 +1863,12 @@ impl fmt::Debug for Expr {
 }
 
 impl Expr {
-    /// Get the [`Dynamic`] value of a constant expression.
+    /// Get the [`Dynamic`] value of a literal constant expression.
     ///
-    /// Returns [`None`] if the expression is not constant.
+    /// Returns [`None`] if the expression is not a literal constant.
     #[inline]
-    pub fn get_constant_value(&self) -> Option<Dynamic> {
+    #[must_use]
+    pub fn get_literal_value(&self) -> Option<Dynamic> {
         Some(match self {
             Self::DynamicConstant(x, _) => x.as_ref().clone(),
             Self::IntegerConstant(x, _) => (*x).into(),
@@ -1845,7 +1883,7 @@ impl Expr {
             Self::Array(x, _) if self.is_constant() => {
                 let mut arr = Array::with_capacity(x.len());
                 arr.extend(x.iter().map(|v| {
-                    v.get_constant_value()
+                    v.get_literal_value()
                         .expect("never fails because a constant array always has a constant value")
                 }));
                 Dynamic::from_array(arr)
@@ -1857,7 +1895,7 @@ impl Expr {
                 x.0.iter().for_each(|(k, v)| {
                     *map.get_mut(k.name.as_str())
                         .expect("never fails because the template should contain all the keys") = v
-                        .get_constant_value()
+                        .get_literal_value()
                         .expect("never fails because a constant map always has a constant value")
                 });
                 Dynamic::from_map(map)
@@ -1866,9 +1904,27 @@ impl Expr {
             _ => return None,
         })
     }
+    /// Create an [`Expr`] from a [`Dynamic`] value.
+    #[inline]
+    #[must_use]
+    pub fn from_dynamic(value: Dynamic, pos: Position) -> Self {
+        match value.0 {
+            Union::Unit(_, _, _) => Self::Unit(pos),
+            Union::Bool(b, _, _) => Self::BoolConstant(b, pos),
+            Union::Str(s, _, _) => Self::StringConstant(s, pos),
+            Union::Char(c, _, _) => Self::CharConstant(c, pos),
+            Union::Int(i, _, _) => Self::IntegerConstant(i, pos),
+
+            #[cfg(not(feature = "no_float"))]
+            Union::Float(f, _, _) => Self::FloatConstant(f, pos),
+
+            _ => Self::DynamicConstant(value.into(), pos),
+        }
+    }
     /// Is the expression a simple variable access?
     #[inline(always)]
-    pub(crate) fn is_variable_access(&self, non_qualified: bool) -> bool {
+    #[must_use]
+    pub(crate) const fn is_variable_access(&self, non_qualified: bool) -> bool {
         match self {
             Self::Variable(_, _, x) => !non_qualified || x.1.is_none(),
             _ => false,
@@ -1876,6 +1932,7 @@ impl Expr {
     }
     /// Return the variable name if the expression a simple variable access.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_variable_name(&self, non_qualified: bool) -> Option<&str> {
         match self {
             Self::Variable(_, _, x) if !non_qualified || x.1.is_none() => Some(x.2.as_str()),
@@ -1884,7 +1941,8 @@ impl Expr {
     }
     /// Get the [position][Position] of the expression.
     #[inline]
-    pub fn position(&self) -> Position {
+    #[must_use]
+    pub const fn position(&self) -> Position {
         match self {
             #[cfg(not(feature = "no_float"))]
             Self::FloatConstant(_, pos) => *pos,
@@ -1898,15 +1956,10 @@ impl Expr {
             | Self::Array(_, pos)
             | Self::Map(_, pos)
             | Self::Variable(_, pos, _)
+            | Self::Stack(_, pos)
             | Self::FnCall(_, pos)
-            | Self::Custom(_, pos) => *pos,
-
-            Self::InterpolatedString(x) => x
-                .first()
-                .expect(
-                    "never fails because an interpolated string always contains at least one item",
-                )
-                .position(),
+            | Self::Custom(_, pos)
+            | Self::InterpolatedString(_, pos) => *pos,
 
             Self::Property(x) => (x.2).1,
             Self::Stmt(x) => x.1,
@@ -1936,14 +1989,10 @@ impl Expr {
             | Self::Dot(_, pos)
             | Self::Index(_, pos)
             | Self::Variable(_, pos, _)
+            | Self::Stack(_, pos)
             | Self::FnCall(_, pos)
-            | Self::Custom(_, pos) => *pos = new_pos,
-
-            Self::InterpolatedString(x) => {
-                x.first_mut()
-                    .expect("never fails because an interpolated string always contains at least one item")
-                    .set_position(new_pos);
-            }
+            | Self::Custom(_, pos)
+            | Self::InterpolatedString(_, pos) => *pos = new_pos,
 
             Self::Property(x) => (x.2).1 = new_pos,
             Self::Stmt(x) => x.1 = new_pos,
@@ -1955,9 +2004,10 @@ impl Expr {
     ///
     /// A pure expression has no side effects.
     #[inline]
+    #[must_use]
     pub fn is_pure(&self) -> bool {
         match self {
-            Self::InterpolatedString(x) | Self::Array(x, _) => x.iter().all(Self::is_pure),
+            Self::InterpolatedString(x, _) | Self::Array(x, _) => x.iter().all(Self::is_pure),
 
             Self::Map(x, _) => x.0.iter().map(|(_, v)| v).all(Self::is_pure),
 
@@ -1965,14 +2015,15 @@ impl Expr {
 
             Self::Stmt(x) => x.0.iter().all(Stmt::is_pure),
 
-            Self::Variable(_, _, _) => true,
+            Self::Variable(_, _, _) | Self::Stack(_, _) => true,
 
             _ => self.is_constant(),
         }
     }
     /// Is the expression the unit `()` literal?
     #[inline(always)]
-    pub fn is_unit(&self) -> bool {
+    #[must_use]
+    pub const fn is_unit(&self) -> bool {
         match self {
             Self::Unit(_) => true,
             _ => false,
@@ -1980,6 +2031,7 @@ impl Expr {
     }
     /// Is the expression a constant?
     #[inline]
+    #[must_use]
     pub fn is_constant(&self) -> bool {
         match self {
             #[cfg(not(feature = "no_float"))]
@@ -1990,9 +2042,10 @@ impl Expr {
             | Self::IntegerConstant(_, _)
             | Self::CharConstant(_, _)
             | Self::StringConstant(_, _)
-            | Self::Unit(_) => true,
+            | Self::Unit(_)
+            | Self::Stack(_, _) => true,
 
-            Self::InterpolatedString(x) | Self::Array(x, _) => x.iter().all(Self::is_constant),
+            Self::InterpolatedString(x, _) | Self::Array(x, _) => x.iter().all(Self::is_constant),
 
             Self::Map(x, _) => x.0.iter().map(|(_, expr)| expr).all(Self::is_constant),
 
@@ -2001,7 +2054,8 @@ impl Expr {
     }
     /// Is a particular [token][Token] allowed as a postfix operator to this expression?
     #[inline]
-    pub fn is_valid_postfix(&self, token: &Token) -> bool {
+    #[must_use]
+    pub const fn is_valid_postfix(&self, token: &Token) -> bool {
         match token {
             #[cfg(not(feature = "no_object"))]
             Token::Period => return true,
@@ -2014,14 +2068,14 @@ impl Expr {
 
             Self::DynamicConstant(_, _)
             | Self::BoolConstant(_, _)
-            | Self::IntegerConstant(_, _)
             | Self::CharConstant(_, _)
             | Self::And(_, _)
             | Self::Or(_, _)
             | Self::Unit(_) => false,
 
-            Self::StringConstant(_, _)
-            | Self::InterpolatedString(_)
+            Self::IntegerConstant(_, _)
+            | Self::StringConstant(_, _)
+            | Self::InterpolatedString(_, _)
             | Self::FnCall(_, _)
             | Self::Stmt(_)
             | Self::Dot(_, _)
@@ -2050,6 +2104,8 @@ impl Expr {
             },
 
             Self::Custom(_, _) => false,
+
+            Self::Stack(_, _) => false,
         }
     }
     /// Recursively walk this expression.
@@ -2074,7 +2130,7 @@ impl Expr {
                     }
                 }
             }
-            Self::InterpolatedString(x) | Self::Array(x, _) => {
+            Self::InterpolatedString(x, _) | Self::Array(x, _) => {
                 for e in x.as_ref() {
                     if !e.walk(path, on_node) {
                         return false;
@@ -2144,7 +2200,7 @@ mod tests {
                 96
             }
         );
-        assert_eq!(size_of::<Scope>(), 160);
+        assert_eq!(size_of::<Scope>(), 464);
         assert_eq!(size_of::<LexError>(), 56);
         assert_eq!(
             size_of::<ParseError>(),
