@@ -1,11 +1,12 @@
 //! Module that defines the extern API of [`Engine`].
 
 use crate::dynamic::Variant;
-use crate::engine::{EvalContext, Imports, State};
-use crate::fn_native::{FnCallArgs, SendSync};
+use crate::engine::{EvalContext, EvalState, Imports};
+use crate::fn_call::FnCallArgs;
+use crate::fn_native::SendSync;
 use crate::fn_register::RegisterNativeFunction;
 use crate::optimize::OptimizationLevel;
-use crate::parser::ParseState;
+use crate::parse::ParseState;
 use crate::{
     scope::Scope, Dynamic, Engine, EvalAltResult, FnAccess, FnNamespace, Identifier, Module,
     NativeCallContext, ParseError, Position, RhaiResult, Shared, AST,
@@ -538,8 +539,9 @@ impl Engine {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
-    /// Indexers for arrays, object maps and strings cannot be registered.
+    /// Panics if the type is [`Array`], [`Map`], [`String`],
+    /// [`ImmutableString`][crate::ImmutableString], `&str` or [`INT`][crate::INT].
+    /// Indexers for arrays, object maps, strings and integers cannot be registered.
     ///
     /// # Example
     ///
@@ -595,6 +597,9 @@ impl Engine {
         {
             panic!("Cannot register indexer for strings.");
         }
+        if TypeId::of::<T>() == TypeId::of::<crate::INT>() {
+            panic!("Cannot register indexer for integers.");
+        }
 
         self.register_fn(crate::engine::FN_IDX_GET, get_fn)
     }
@@ -606,8 +611,9 @@ impl Engine {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
-    /// Indexers for arrays, object maps and strings cannot be registered.
+    /// Panics if the type is [`Array`], [`Map`], [`String`],
+    /// [`ImmutableString`][crate::ImmutableString], `&str` or [`INT`][crate::INT].
+    /// Indexers for arrays, object maps, strings and integers cannot be registered.
     ///
     /// # Example
     ///
@@ -669,6 +675,9 @@ impl Engine {
         {
             panic!("Cannot register indexer for strings.");
         }
+        if TypeId::of::<T>() == TypeId::of::<crate::INT>() {
+            panic!("Cannot register indexer for integers.");
+        }
 
         self.register_result_fn(crate::engine::FN_IDX_GET, get_fn)
     }
@@ -678,8 +687,9 @@ impl Engine {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
-    /// Indexers for arrays, object maps and strings cannot be registered.
+    /// Panics if the type is [`Array`], [`Map`], [`String`],
+    /// [`ImmutableString`][crate::ImmutableString], `&str` or [`INT`][crate::INT].
+    /// Indexers for arrays, object maps, strings and integers cannot be registered.
     ///
     /// # Example
     ///
@@ -737,6 +747,9 @@ impl Engine {
         {
             panic!("Cannot register indexer for strings.");
         }
+        if TypeId::of::<T>() == TypeId::of::<crate::INT>() {
+            panic!("Cannot register indexer for integers.");
+        }
 
         self.register_fn(crate::engine::FN_IDX_SET, set_fn)
     }
@@ -746,8 +759,9 @@ impl Engine {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
-    /// Indexers for arrays, object maps and strings cannot be registered.
+    /// Panics if the type is [`Array`], [`Map`], [`String`],
+    /// [`ImmutableString`][crate::ImmutableString], `&str` or [`INT`][crate::INT].
+    /// Indexers for arrays, object maps, strings and integers cannot be registered.
     ///
     /// # Example
     ///
@@ -812,6 +826,9 @@ impl Engine {
         {
             panic!("Cannot register indexer for strings.");
         }
+        if TypeId::of::<T>() == TypeId::of::<crate::INT>() {
+            panic!("Cannot register indexer for integers.");
+        }
 
         self.register_result_fn(crate::engine::FN_IDX_SET, set_fn)
     }
@@ -821,8 +838,9 @@ impl Engine {
     ///
     /// # Panics
     ///
-    /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
-    /// Indexers for arrays, object maps and strings cannot be registered.
+    /// Panics if the type is [`Array`], [`Map`], [`String`],
+    /// [`ImmutableString`][crate::ImmutableString], `&str` or [`INT`][crate::INT].
+    /// Indexers for arrays, object maps, strings and integers cannot be registered.
     ///
     /// # Example
     ///
@@ -885,14 +903,6 @@ impl Engine {
         self.global_modules.insert(0, module);
         self
     }
-    /// Register a shared [`Module`] into the global namespace of [`Engine`].
-    /// This function is deprecated and will be removed in the future.
-    /// Use [`register_global_module`][Engine::register_global_module] instead.
-    #[inline(always)]
-    #[deprecated(since = "0.19.9", note = "use `register_global_module` instead")]
-    pub fn load_package(&mut self, module: impl Into<Shared<Module>>) -> &mut Self {
-        self.register_global_module(module.into())
-    }
     /// Register a shared [`Module`] as a static module namespace with the [`Engine`].
     ///
     /// Functions marked [`FnNamespace::Global`] and type iterators are exposed to scripts without
@@ -981,19 +991,6 @@ impl Engine {
         self
     }
 
-    /// Register a shared [`Module`] as a static module namespace with the [`Engine`].
-    /// This function is deprecated and will be removed in the future.
-    /// Use [`register_static_module`][Engine::register_static_module] instead.
-    #[cfg(not(feature = "no_module"))]
-    #[inline(always)]
-    #[deprecated(since = "0.19.9", note = "use `register_static_module` instead")]
-    pub fn register_module(
-        &mut self,
-        name: impl AsRef<str> + Into<Identifier>,
-        module: impl Into<Shared<Module>>,
-    ) -> &mut Self {
-        self.register_static_module(name, module.into())
-    }
     /// Compile a string into an [`AST`], which can be used later for evaluation.
     ///
     /// # Example
@@ -1014,6 +1011,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn compile(&self, script: &str) -> Result<AST, ParseError> {
         self.compile_with_scope(&Default::default(), script)
     }
@@ -1056,6 +1054,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn compile_with_scope(&self, scope: &Scope, script: &str) -> Result<AST, ParseError> {
         self.compile_scripts_with_scope(scope, &[script])
     }
@@ -1069,6 +1068,7 @@ impl Engine {
     /// [`AST`]. When it is evaluated later, `import` statement directly recall pre-resolved
     /// [modules][Module] and the resolution process is not performed again.
     #[cfg(not(feature = "no_module"))]
+    #[must_use]
     pub fn compile_into_self_contained(
         &self,
         scope: &Scope,
@@ -1101,38 +1101,34 @@ impl Engine {
             });
         }
 
-        let mut resolver = StaticModuleResolver::new();
         let mut ast = self.compile_scripts_with_scope(scope, &[script])?;
-        let mut imports = Default::default();
 
-        collect_imports(&ast, &mut resolver, &mut imports);
+        if let Some(ref module_resolver) = self.module_resolver {
+            let mut resolver = StaticModuleResolver::new();
+            let mut imports = Default::default();
 
-        if !imports.is_empty() {
-            while let Some(path) = imports.iter().next() {
-                let path = path.clone();
+            collect_imports(&ast, &mut resolver, &mut imports);
 
-                match self
-                    .module_resolver
-                    .resolve_ast(self, None, &path, Position::NONE)
-                {
-                    Some(Ok(module_ast)) => {
-                        collect_imports(&module_ast, &mut resolver, &mut imports)
+            if !imports.is_empty() {
+                while let Some(path) = imports.iter().next() {
+                    let path = path.clone();
+
+                    match module_resolver.resolve_ast(self, None, &path, Position::NONE) {
+                        Some(Ok(module_ast)) => {
+                            collect_imports(&module_ast, &mut resolver, &mut imports)
+                        }
+                        Some(err) => return err,
+                        None => (),
                     }
-                    Some(err) => return err,
-                    None => (),
+
+                    let module = module_resolver.resolve(self, None, &path, Position::NONE)?;
+                    let module = shared_take_or_clone(module);
+
+                    imports.remove(&path);
+                    resolver.insert(path, module);
                 }
-
-                let module = shared_take_or_clone(self.module_resolver.resolve(
-                    self,
-                    None,
-                    &path,
-                    Position::NONE,
-                )?);
-
-                imports.remove(&path);
-                resolver.insert(path, module);
+                ast.set_resolver(resolver);
             }
-            ast.set_resolver(resolver);
         }
 
         Ok(ast)
@@ -1184,6 +1180,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn compile_scripts_with_scope(
         &self,
         scope: &Scope,
@@ -1193,6 +1190,7 @@ impl Engine {
     }
     /// Join a list of strings and compile into an [`AST`] using own scope at a specific optimization level.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn compile_with_scope_and_optimization_level(
         &self,
         scope: &Scope,
@@ -1211,6 +1209,7 @@ impl Engine {
     /// Read the contents of a file into a string.
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
+    #[must_use]
     fn read_file(path: std::path::PathBuf) -> Result<String, Box<EvalAltResult>> {
         use std::io::Read;
 
@@ -1266,6 +1265,7 @@ impl Engine {
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
+    #[must_use]
     pub fn compile_file(&self, path: std::path::PathBuf) -> Result<AST, Box<EvalAltResult>> {
         self.compile_file_with_scope(&Default::default(), path)
     }
@@ -1305,6 +1305,7 @@ impl Engine {
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
+    #[must_use]
     pub fn compile_file_with_scope(
         &self,
         scope: &Scope,
@@ -1313,7 +1314,8 @@ impl Engine {
         Self::read_file(path).and_then(|contents| Ok(self.compile_with_scope(scope, &contents)?))
     }
     /// Parse a JSON string into an [object map][`Map`].
-    /// This is a light-weight alternative to using, say, [`serde_json`] to deserialize the JSON.
+    /// This is a light-weight alternative to using, say,
+    /// [`serde_json`](https://crates.io/crates/serde_json) to deserialize the JSON.
     ///
     /// Not available under `no_object`.
     ///
@@ -1341,7 +1343,8 @@ impl Engine {
     ///
     /// let map = engine.parse_json(
     ///     r#"{"a":123, "b":42, "c":{"x":false, "y":true}, "d":null}"#
-    ///         .replace("{", "#{").as_str(), true)?;
+    ///         .replace("{", "#{").as_str(),
+    /// true)?;
     ///
     /// assert_eq!(map.len(), 4);
     /// assert_eq!(map["a"].as_int().unwrap(), 123);
@@ -1354,6 +1357,7 @@ impl Engine {
     /// # }
     /// ```
     #[cfg(not(feature = "no_object"))]
+    #[must_use]
     pub fn parse_json(
         &self,
         json: impl AsRef<str>,
@@ -1429,6 +1433,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn compile_expression(&self, script: &str) -> Result<AST, ParseError> {
         self.compile_expression_with_scope(&Default::default(), script)
     }
@@ -1472,6 +1477,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn compile_expression_with_scope(
         &self,
         scope: &Scope,
@@ -1504,6 +1510,7 @@ impl Engine {
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
+    #[must_use]
     pub fn eval_file<T: Variant + Clone>(
         &self,
         path: std::path::PathBuf,
@@ -1534,6 +1541,7 @@ impl Engine {
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
+    #[must_use]
     pub fn eval_file_with_scope<T: Variant + Clone>(
         &self,
         scope: &mut Scope,
@@ -1556,6 +1564,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval<T: Variant + Clone>(&self, script: &str) -> Result<T, Box<EvalAltResult>> {
         self.eval_with_scope(&mut Default::default(), script)
     }
@@ -1582,6 +1591,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval_with_scope<T: Variant + Clone>(
         &self,
         scope: &mut Scope,
@@ -1609,6 +1619,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval_expression<T: Variant + Clone>(
         &self,
         script: &str,
@@ -1634,6 +1645,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval_expression_with_scope<T: Variant + Clone>(
         &self,
         scope: &mut Scope,
@@ -1672,6 +1684,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval_ast<T: Variant + Clone>(&self, ast: &AST) -> Result<T, Box<EvalAltResult>> {
         self.eval_ast_with_scope(&mut Default::default(), ast)
     }
@@ -1705,6 +1718,7 @@ impl Engine {
     /// # }
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn eval_ast_with_scope<T: Variant + Clone>(
         &self,
         scope: &mut Scope,
@@ -1727,6 +1741,7 @@ impl Engine {
     }
     /// Evaluate an [`AST`] with own scope.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn eval_ast_with_scope_raw<'a>(
         &self,
         scope: &mut Scope,
@@ -1734,8 +1749,8 @@ impl Engine {
         ast: &'a AST,
         level: usize,
     ) -> RhaiResult {
-        let mut state: State = Default::default();
-        state.source = ast.clone_source();
+        let mut state = EvalState::new();
+        state.source = ast.source_raw().cloned();
         #[cfg(not(feature = "no_module"))]
         {
             state.resolver = ast.resolver();
@@ -1816,8 +1831,8 @@ impl Engine {
         ast: &AST,
     ) -> Result<(), Box<EvalAltResult>> {
         let mods = &mut Default::default();
-        let mut state: State = Default::default();
-        state.source = ast.clone_source();
+        let mut state = EvalState::new();
+        state.source = ast.source_raw().cloned();
         #[cfg(not(feature = "no_module"))]
         {
             state.resolver = ast.resolver();
@@ -1875,12 +1890,13 @@ impl Engine {
     /// ```
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn call_fn<T: Variant + Clone>(
         &self,
         scope: &mut Scope,
         ast: &AST,
         name: impl AsRef<str>,
-        args: impl crate::fn_args::FuncArgs,
+        args: impl crate::FuncArgs,
     ) -> Result<T, Box<EvalAltResult>> {
         let mut arg_values: crate::StaticVec<_> = Default::default();
         args.parse(&mut arg_values);
@@ -1890,14 +1906,14 @@ impl Engine {
 
         let typ = self.map_type_name(result.type_name());
 
-        return result.try_cast().ok_or_else(|| {
+        result.try_cast().ok_or_else(|| {
             EvalAltResult::ErrorMismatchOutputType(
                 self.map_type_name(type_name::<T>()).into(),
                 typ.into(),
                 Position::NONE,
             )
             .into()
-        });
+        })
     }
     /// Call a script function defined in an [`AST`] with multiple [`Dynamic`] arguments
     /// and optionally a value for binding to the `this` pointer.
@@ -1954,6 +1970,7 @@ impl Engine {
     /// ```
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn call_fn_dynamic(
         &self,
         scope: &mut Scope,
@@ -1977,6 +1994,7 @@ impl Engine {
     /// clone them _before_ calling this function.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn call_fn_dynamic_raw(
         &self,
         scope: &mut Scope,
@@ -1986,7 +2004,7 @@ impl Engine {
         this_ptr: &mut Option<&mut Dynamic>,
         args: &mut FnCallArgs,
     ) -> RhaiResult {
-        let state = &mut Default::default();
+        let state = &mut EvalState::new();
         let mods = &mut Default::default();
         let lib = &[ast.lib()];
         let statements = ast.statements();
@@ -2034,6 +2052,7 @@ impl Engine {
     /// Then, the [`AST`] is cloned and the copy re-optimized before running.
     #[cfg(not(feature = "no_optimize"))]
     #[inline(always)]
+    #[must_use]
     pub fn optimize_ast(
         &self,
         scope: &Scope,
@@ -2045,7 +2064,12 @@ impl Engine {
             .lib()
             .iter_fn()
             .filter(|f| f.func.is_script())
-            .map(|f| f.func.get_fn_def().clone())
+            .map(|f| {
+                f.func
+                    .get_script_fn_def()
+                    .expect("never fails because the function is scripted")
+                    .clone()
+            })
             .collect();
 
         #[cfg(feature = "no_function")]
@@ -2062,6 +2086,7 @@ impl Engine {
     /// 2) Functions in registered sub-modules
     /// 3) Functions in packages (optional)
     #[cfg(feature = "metadata")]
+    #[must_use]
     pub fn gen_fn_signatures(&self, include_packages: bool) -> Vec<String> {
         let mut signatures: Vec<_> = Default::default();
 
@@ -2194,7 +2219,7 @@ impl Engine {
     /// ```
     #[inline(always)]
     pub fn on_print(&mut self, callback: impl Fn(&str) + SendSync + 'static) -> &mut Self {
-        self.print = Box::new(callback);
+        self.print = Some(Box::new(callback));
         self
     }
     /// Override default action of `debug` (print to stdout using [`println!`])
@@ -2233,7 +2258,7 @@ impl Engine {
         &mut self,
         callback: impl Fn(&str, Option<&str>, Position) + SendSync + 'static,
     ) -> &mut Self {
-        self.debug = Box::new(callback);
+        self.debug = Some(Box::new(callback));
         self
     }
 }
