@@ -28,7 +28,8 @@ impl Engine {
     /// Not available under `no_optimize`.
     #[cfg(not(feature = "no_optimize"))]
     #[inline(always)]
-    pub fn optimization_level(&self) -> crate::OptimizationLevel {
+    #[must_use]
+    pub const fn optimization_level(&self) -> crate::OptimizationLevel {
         self.optimization_level
     }
     /// Set the maximum levels of function calls allowed for a script in order to avoid
@@ -48,7 +49,8 @@ impl Engine {
     #[cfg(not(feature = "unchecked"))]
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
-    pub fn max_call_levels(&self) -> usize {
+    #[must_use]
+    pub const fn max_call_levels(&self) -> usize {
         self.limits.max_call_stack_depth
     }
     /// Set the maximum number of operations allowed for a script to run to avoid
@@ -66,8 +68,13 @@ impl Engine {
     /// Not available under `unchecked`.
     #[cfg(not(feature = "unchecked"))]
     #[inline(always)]
-    pub fn max_operations(&self) -> u64 {
-        self.limits.max_operations.map_or(0, NonZeroU64::get)
+    #[must_use]
+    pub const fn max_operations(&self) -> u64 {
+        if let Some(n) = self.limits.max_operations {
+            n.get()
+        } else {
+            0
+        }
     }
     /// Set the maximum number of imported [modules][crate::Module] allowed for a script.
     ///
@@ -85,7 +92,8 @@ impl Engine {
     #[cfg(not(feature = "unchecked"))]
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
-    pub fn max_modules(&self) -> usize {
+    #[must_use]
+    pub const fn max_modules(&self) -> usize {
         self.limits.max_modules
     }
     /// Set the depth limits for expressions (0 for unlimited).
@@ -110,8 +118,13 @@ impl Engine {
     /// Not available under `unchecked`.
     #[cfg(not(feature = "unchecked"))]
     #[inline(always)]
-    pub fn max_expr_depth(&self) -> usize {
-        self.limits.max_expr_depth.map_or(0, NonZeroUsize::get)
+    #[must_use]
+    pub const fn max_expr_depth(&self) -> usize {
+        if let Some(n) = self.limits.max_expr_depth {
+            n.get()
+        } else {
+            0
+        }
     }
     /// The depth limit for expressions in functions (0 for unlimited).
     ///
@@ -119,10 +132,13 @@ impl Engine {
     #[cfg(not(feature = "unchecked"))]
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
-    pub fn max_function_expr_depth(&self) -> usize {
-        self.limits
-            .max_function_expr_depth
-            .map_or(0, NonZeroUsize::get)
+    #[must_use]
+    pub const fn max_function_expr_depth(&self) -> usize {
+        if let Some(n) = self.limits.max_function_expr_depth {
+            n.get()
+        } else {
+            0
+        }
     }
     /// Set the maximum length of [strings][crate::ImmutableString] (0 for unlimited).
     ///
@@ -138,8 +154,13 @@ impl Engine {
     /// Not available under `unchecked`.
     #[cfg(not(feature = "unchecked"))]
     #[inline(always)]
-    pub fn max_string_size(&self) -> usize {
-        self.limits.max_string_size.map_or(0, NonZeroUsize::get)
+    #[must_use]
+    pub const fn max_string_size(&self) -> usize {
+        if let Some(n) = self.limits.max_string_size {
+            n.get()
+        } else {
+            0
+        }
     }
     /// Set the maximum length of [arrays][crate::Array] (0 for unlimited).
     ///
@@ -157,8 +178,13 @@ impl Engine {
     #[cfg(not(feature = "unchecked"))]
     #[cfg(not(feature = "no_index"))]
     #[inline(always)]
-    pub fn max_array_size(&self) -> usize {
-        self.limits.max_array_size.map_or(0, NonZeroUsize::get)
+    #[must_use]
+    pub const fn max_array_size(&self) -> usize {
+        if let Some(n) = self.limits.max_array_size {
+            n.get()
+        } else {
+            0
+        }
     }
     /// Set the maximum size of [object maps][crate::Map] (0 for unlimited).
     ///
@@ -176,8 +202,13 @@ impl Engine {
     #[cfg(not(feature = "unchecked"))]
     #[cfg(not(feature = "no_object"))]
     #[inline(always)]
-    pub fn max_map_size(&self) -> usize {
-        self.limits.max_map_size.map_or(0, NonZeroUsize::get)
+    #[must_use]
+    pub const fn max_map_size(&self) -> usize {
+        if let Some(n) = self.limits.max_map_size {
+            n.get()
+        } else {
+            0
+        }
     }
     /// Set the module resolution service used by the [`Engine`].
     ///
@@ -188,7 +219,7 @@ impl Engine {
         &mut self,
         resolver: impl crate::ModuleResolver + 'static,
     ) -> &mut Self {
-        self.module_resolver = Box::new(resolver);
+        self.module_resolver = Some(Box::new(resolver));
         self
     }
     /// Disable a particular keyword or operator in the language.
@@ -260,6 +291,7 @@ impl Engine {
     /// # Ok(())
     /// # }
     /// ```
+    #[must_use]
     pub fn register_custom_operator(
         &mut self,
         keyword: impl AsRef<str> + Into<Identifier>,
