@@ -2,20 +2,14 @@
 
 use crate::ast::{FnAccess, FnCallHashes};
 use crate::engine::Imports;
+use crate::fn_call::FnCallArgs;
 use crate::plugin::PluginFunction;
-use crate::token::is_valid_identifier;
 use crate::{
-    calc_fn_hash, Dynamic, Engine, EvalAltResult, EvalContext, Identifier, Module, Position,
-    RhaiResult, StaticVec,
+    calc_fn_hash, Dynamic, Engine, EvalAltResult, EvalContext, Module, Position, RhaiResult,
 };
+use std::fmt;
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
-use std::{
-    convert::{TryFrom, TryInto},
-    fmt,
-    iter::once,
-    mem,
-};
 
 /// Trait that maps to `Send + Sync` only under the `sync` feature.
 #[cfg(feature = "sync")]
@@ -94,7 +88,8 @@ impl<'a, M: AsRef<[&'a Module]> + ?Sized> From<(&'a Engine, &'a str, &'a M)>
 impl<'a> NativeCallContext<'a> {
     /// Create a new [`NativeCallContext`].
     #[inline(always)]
-    pub fn new(engine: &'a Engine, fn_name: &'a str, lib: &'a [&Module]) -> Self {
+    #[must_use]
+    pub const fn new(engine: &'a Engine, fn_name: &'a str, lib: &'a [&Module]) -> Self {
         Self {
             engine,
             fn_name,
@@ -110,34 +105,38 @@ impl<'a> NativeCallContext<'a> {
     #[cfg(feature = "internals")]
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
-    pub fn new_with_all_fields(
+    #[must_use]
+    pub const fn new_with_all_fields(
         engine: &'a Engine,
         fn_name: &'a str,
-        source: &'a Option<&str>,
+        source: Option<&'a str>,
         imports: &'a Imports,
         lib: &'a [&Module],
     ) -> Self {
         Self {
             engine,
             fn_name,
-            source: source.clone(),
+            source,
             mods: Some(imports),
             lib,
         }
     }
     /// The current [`Engine`].
     #[inline(always)]
-    pub fn engine(&self) -> &Engine {
+    #[must_use]
+    pub const fn engine(&self) -> &Engine {
         self.engine
     }
     /// Name of the function called.
     #[inline(always)]
-    pub fn fn_name(&self) -> &str {
+    #[must_use]
+    pub const fn fn_name(&self) -> &str {
         self.fn_name
     }
     /// The current source.
     #[inline(always)]
-    pub fn source(&self) -> Option<&str> {
+    #[must_use]
+    pub const fn source(&self) -> Option<&str> {
         self.source
     }
     /// Get an iterator over the current set of modules imported via `import` statements.
@@ -145,6 +144,7 @@ impl<'a> NativeCallContext<'a> {
     /// Not available under `no_module`.
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
+    #[must_use]
     pub fn iter_imports(&self) -> impl Iterator<Item = (&str, &Module)> {
         self.mods.iter().flat_map(|&m| m.iter())
     }
@@ -152,6 +152,7 @@ impl<'a> NativeCallContext<'a> {
     #[cfg(not(feature = "no_module"))]
     #[allow(dead_code)]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn iter_imports_raw(
         &self,
     ) -> impl Iterator<Item = (&crate::Identifier, &Shared<Module>)> {
@@ -164,11 +165,13 @@ impl<'a> NativeCallContext<'a> {
     #[cfg(feature = "internals")]
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
-    pub fn imports(&self) -> Option<&Imports> {
+    #[must_use]
+    pub const fn imports(&self) -> Option<&Imports> {
         self.mods
     }
     /// Get an iterator over the namespaces containing definitions of all script-defined functions.
     #[inline(always)]
+    #[must_use]
     pub fn iter_namespaces(&self) -> impl Iterator<Item = &Module> {
         self.lib.iter().cloned()
     }
@@ -176,7 +179,8 @@ impl<'a> NativeCallContext<'a> {
     /// Exported under the `internals` feature only.
     #[cfg(feature = "internals")]
     #[inline(always)]
-    pub fn namespaces(&self) -> &[&Module] {
+    #[must_use]
+    pub const fn namespaces(&self) -> &[&Module] {
         self.lib
     }
     /// Call a function inside the call context.
@@ -192,15 +196,16 @@ impl<'a> NativeCallContext<'a> {
     /// If `is_method` is [`true`], the first argument is assumed to be passed
     /// by reference and is not consumed.
     #[inline(always)]
+    #[must_use]
     pub fn call_fn_dynamic_raw(
         &self,
         fn_name: impl AsRef<str>,
-        is_method: bool,
+        is_method_call: bool,
         args: &mut [&mut Dynamic],
     ) -> RhaiResult {
         let fn_name = fn_name.as_ref();
 
-        let hash = if is_method {
+        let hash = if is_method_call {
             FnCallHashes::from_script_and_native(
                 calc_fn_hash(fn_name, args.len() - 1),
                 calc_fn_hash(fn_name, args.len()),
@@ -217,8 +222,8 @@ impl<'a> NativeCallContext<'a> {
                 fn_name,
                 hash,
                 args,
-                is_method,
-                is_method,
+                is_method_call,
+                is_method_call,
                 Position::NONE,
                 None,
                 0,
@@ -230,18 +235,21 @@ impl<'a> NativeCallContext<'a> {
 /// Consume a [`Shared`] resource and return a mutable reference to the wrapped value.
 /// If the resource is shared (i.e. has other outstanding references), a cloned copy is used.
 #[inline(always)]
+#[must_use]
 pub fn shared_make_mut<T: Clone>(value: &mut Shared<T>) -> &mut T {
     Shared::make_mut(value)
 }
 
 /// Consume a [`Shared`] resource if is unique (i.e. not shared), or clone it otherwise.
 #[inline(always)]
+#[must_use]
 pub fn shared_take_or_clone<T: Clone>(value: Shared<T>) -> T {
     shared_try_take(value).unwrap_or_else(|v| v.as_ref().clone())
 }
 
 /// Consume a [`Shared`] resource if is unique (i.e. not shared).
 #[inline(always)]
+#[must_use]
 pub fn shared_try_take<T>(value: Shared<T>) -> Result<T, Shared<T>> {
     Shared::try_unwrap(value)
 }
@@ -252,171 +260,13 @@ pub fn shared_try_take<T>(value: Shared<T>) -> Result<T, Shared<T>> {
 ///
 /// Panics if the resource is shared (i.e. has other outstanding references).
 #[inline(always)]
+#[must_use]
 pub fn shared_take<T>(value: Shared<T>) -> T {
     shared_try_take(value)
         .ok()
         .expect("resource should have no outstanding references")
 }
 
-/// Arguments to a function call, which is a list of [`&mut Dynamic`][Dynamic].
-pub type FnCallArgs<'a> = [&'a mut Dynamic];
-
-/// A general function pointer, which may carry additional (i.e. curried) argument values
-/// to be passed onto a function during a call.
-#[derive(Debug, Clone)]
-pub struct FnPtr(Identifier, StaticVec<Dynamic>);
-
-impl FnPtr {
-    /// Create a new function pointer.
-    #[inline(always)]
-    pub fn new(name: impl Into<Identifier>) -> Result<Self, Box<EvalAltResult>> {
-        name.into().try_into()
-    }
-    /// Create a new function pointer without checking its parameters.
-    #[inline(always)]
-    pub(crate) fn new_unchecked(name: Identifier, curry: StaticVec<Dynamic>) -> Self {
-        Self(name.into(), curry)
-    }
-    /// Get the name of the function.
-    #[inline(always)]
-    pub fn fn_name(&self) -> &str {
-        self.get_fn_name().as_ref()
-    }
-    /// Get the name of the function.
-    #[inline(always)]
-    pub(crate) fn get_fn_name(&self) -> &Identifier {
-        &self.0
-    }
-    /// Get the underlying data of the function pointer.
-    #[inline(always)]
-    pub(crate) fn take_data(self) -> (Identifier, StaticVec<Dynamic>) {
-        (self.0, self.1)
-    }
-    /// Get the curried arguments.
-    #[inline(always)]
-    pub fn curry(&self) -> &[Dynamic] {
-        self.1.as_ref()
-    }
-    /// Add a new curried argument.
-    #[inline(always)]
-    pub fn add_curry(&mut self, value: Dynamic) -> &mut Self {
-        self.1.push(value);
-        self
-    }
-    /// Set curried arguments to the function pointer.
-    #[inline(always)]
-    pub fn set_curry(&mut self, values: impl IntoIterator<Item = Dynamic>) -> &mut Self {
-        self.1 = values.into_iter().collect();
-        self
-    }
-    /// Is the function pointer curried?
-    #[inline(always)]
-    pub fn is_curried(&self) -> bool {
-        !self.1.is_empty()
-    }
-    /// Does the function pointer refer to an anonymous function?
-    ///
-    /// Not available under `no_function`.
-    #[cfg(not(feature = "no_function"))]
-    #[inline(always)]
-    pub fn is_anonymous(&self) -> bool {
-        self.0.starts_with(crate::engine::FN_ANONYMOUS)
-    }
-    /// Call the function pointer with curried arguments (if any).
-    ///
-    /// If this function is a script-defined function, it must not be marked private.
-    ///
-    /// # WARNING
-    ///
-    /// All the arguments are _consumed_, meaning that they're replaced by `()`.
-    /// This is to avoid unnecessarily cloning the arguments.
-    /// Do not use the arguments after this call. If they are needed afterwards,
-    /// clone them _before_ calling this function.
-    #[inline(always)]
-    pub fn call_dynamic(
-        &self,
-        ctx: &NativeCallContext,
-        this_ptr: Option<&mut Dynamic>,
-        mut arg_values: impl AsMut<[Dynamic]>,
-    ) -> RhaiResult {
-        let mut args_data;
-
-        let arg_values = if self.curry().is_empty() {
-            arg_values.as_mut()
-        } else {
-            args_data = self
-                .curry()
-                .iter()
-                .cloned()
-                .chain(arg_values.as_mut().iter_mut().map(mem::take))
-                .collect::<StaticVec<_>>();
-
-            args_data.as_mut()
-        };
-
-        let is_method = this_ptr.is_some();
-
-        let mut args: StaticVec<_> = if let Some(obj) = this_ptr {
-            once(obj).chain(arg_values.iter_mut()).collect()
-        } else {
-            arg_values.iter_mut().collect()
-        };
-
-        ctx.call_fn_dynamic_raw(self.fn_name(), is_method, &mut args)
-    }
-}
-
-impl fmt::Display for FnPtr {
-    #[inline(always)]
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "Fn({})", self.0)
-    }
-}
-
-impl TryFrom<Identifier> for FnPtr {
-    type Error = Box<EvalAltResult>;
-
-    #[inline(always)]
-    fn try_from(value: Identifier) -> Result<Self, Self::Error> {
-        if is_valid_identifier(value.chars()) {
-            Ok(Self(value, Default::default()))
-        } else {
-            EvalAltResult::ErrorFunctionNotFound(value.to_string(), Position::NONE).into()
-        }
-    }
-}
-
-#[cfg(not(feature = "no_smartstring"))]
-impl TryFrom<crate::ImmutableString> for FnPtr {
-    type Error = Box<EvalAltResult>;
-
-    #[inline(always)]
-    fn try_from(value: crate::ImmutableString) -> Result<Self, Self::Error> {
-        let s: Identifier = value.into();
-        Self::try_from(s)
-    }
-}
-
-impl TryFrom<String> for FnPtr {
-    type Error = Box<EvalAltResult>;
-
-    #[inline(always)]
-    fn try_from(value: String) -> Result<Self, Self::Error> {
-        let s: Identifier = value.into();
-        Self::try_from(s)
-    }
-}
-
-impl TryFrom<&str> for FnPtr {
-    type Error = Box<EvalAltResult>;
-
-    #[inline(always)]
-    fn try_from(value: &str) -> Result<Self, Self::Error> {
-        let s: Identifier = value.into();
-        Self::try_from(s)
-    }
-}
-
 /// A general function trail object.
 #[cfg(not(feature = "sync"))]
 pub type FnAny = dyn Fn(NativeCallContext, &mut FnCallArgs) -> RhaiResult;
@@ -520,6 +370,7 @@ impl fmt::Display for CallableFunction {
 impl CallableFunction {
     /// Is this a pure native Rust function?
     #[inline(always)]
+    #[must_use]
     pub fn is_pure(&self) -> bool {
         match self {
             Self::Pure(_) => true,
@@ -533,6 +384,7 @@ impl CallableFunction {
     }
     /// Is this a native Rust method function?
     #[inline(always)]
+    #[must_use]
     pub fn is_method(&self) -> bool {
         match self {
             Self::Method(_) => true,
@@ -546,7 +398,8 @@ impl CallableFunction {
     }
     /// Is this an iterator function?
     #[inline(always)]
-    pub fn is_iter(&self) -> bool {
+    #[must_use]
+    pub const fn is_iter(&self) -> bool {
         match self {
             Self::Iterator(_) => true,
             Self::Pure(_) | Self::Method(_) | Self::Plugin(_) => false,
@@ -557,7 +410,8 @@ impl CallableFunction {
     }
     /// Is this a Rhai-scripted function?
     #[inline(always)]
-    pub fn is_script(&self) -> bool {
+    #[must_use]
+    pub const fn is_script(&self) -> bool {
         match self {
             #[cfg(not(feature = "no_function"))]
             Self::Script(_) => true,
@@ -567,7 +421,8 @@ impl CallableFunction {
     }
     /// Is this a plugin function?
     #[inline(always)]
-    pub fn is_plugin_fn(&self) -> bool {
+    #[must_use]
+    pub const fn is_plugin_fn(&self) -> bool {
         match self {
             Self::Plugin(_) => true,
             Self::Pure(_) | Self::Method(_) | Self::Iterator(_) => false,
@@ -578,7 +433,8 @@ impl CallableFunction {
     }
     /// Is this a native Rust function?
     #[inline(always)]
-    pub fn is_native(&self) -> bool {
+    #[must_use]
+    pub const fn is_native(&self) -> bool {
         match self {
             Self::Pure(_) | Self::Method(_) => true,
             Self::Plugin(_) => true,
@@ -590,6 +446,7 @@ impl CallableFunction {
     }
     /// Get the access mode.
     #[inline(always)]
+    #[must_use]
     pub fn access(&self) -> FnAccess {
         match self {
             Self::Plugin(_) => FnAccess::Public,
@@ -600,84 +457,68 @@ impl CallableFunction {
         }
     }
     /// Get a shared reference to a native Rust function.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the [`CallableFunction`] is not [`Pure`][CallableFunction::Pure] or
-    /// [`Method`][CallableFunction::Method].
     #[inline(always)]
-    pub fn get_native_fn(&self) -> &Shared<FnAny> {
+    #[must_use]
+    pub fn get_native_fn(&self) -> Option<&Shared<FnAny>> {
         match self {
-            Self::Pure(f) | Self::Method(f) => f,
-            Self::Iterator(_) | Self::Plugin(_) => panic!("function should be native"),
+            Self::Pure(f) | Self::Method(f) => Some(f),
+            Self::Iterator(_) | Self::Plugin(_) => None,
 
             #[cfg(not(feature = "no_function"))]
-            Self::Script(_) => panic!("function should be native"),
+            Self::Script(_) => None,
         }
     }
     /// Get a shared reference to a script-defined function definition.
     ///
     /// Not available under `no_function`.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the [`CallableFunction`] is not [`Script`][CallableFunction::Script].
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
-    pub fn get_fn_def(&self) -> &Shared<crate::ast::ScriptFnDef> {
+    #[must_use]
+    pub const fn get_script_fn_def(&self) -> Option<&Shared<crate::ast::ScriptFnDef>> {
         match self {
-            Self::Pure(_) | Self::Method(_) | Self::Iterator(_) | Self::Plugin(_) => {
-                panic!("function should be scripted")
-            }
-            Self::Script(f) => f,
+            Self::Pure(_) | Self::Method(_) | Self::Iterator(_) | Self::Plugin(_) => None,
+            Self::Script(f) => Some(f),
         }
     }
     /// Get a reference to an iterator function.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the [`CallableFunction`] is not [`Iterator`][CallableFunction::Iterator].
     #[inline(always)]
-    pub fn get_iter_fn(&self) -> IteratorFn {
+    #[must_use]
+    pub fn get_iter_fn(&self) -> Option<IteratorFn> {
         match self {
-            Self::Iterator(f) => *f,
-            Self::Pure(_) | Self::Method(_) | Self::Plugin(_) => {
-                panic!("function should an iterator")
-            }
+            Self::Iterator(f) => Some(*f),
+            Self::Pure(_) | Self::Method(_) | Self::Plugin(_) => None,
 
             #[cfg(not(feature = "no_function"))]
-            Self::Script(_) => panic!("function should be an iterator"),
+            Self::Script(_) => None,
         }
     }
     /// Get a shared reference to a plugin function.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the [`CallableFunction`] is not [`Plugin`][CallableFunction::Plugin].
     #[inline(always)]
-    pub fn get_plugin_fn<'s>(&'s self) -> &Shared<FnPlugin> {
+    #[must_use]
+    pub fn get_plugin_fn<'s>(&'s self) -> Option<&Shared<FnPlugin>> {
         match self {
-            Self::Plugin(f) => f,
-            Self::Pure(_) | Self::Method(_) | Self::Iterator(_) => {
-                panic!("function should a plugin")
-            }
+            Self::Plugin(f) => Some(f),
+            Self::Pure(_) | Self::Method(_) | Self::Iterator(_) => None,
 
             #[cfg(not(feature = "no_function"))]
-            Self::Script(_) => panic!("function should a plugin"),
+            Self::Script(_) => None,
         }
     }
     /// Create a new [`CallableFunction::Pure`].
     #[inline(always)]
+    #[must_use]
     pub fn from_pure(func: Box<FnAny>) -> Self {
         Self::Pure(func.into())
     }
     /// Create a new [`CallableFunction::Method`].
     #[inline(always)]
+    #[must_use]
     pub fn from_method(func: Box<FnAny>) -> Self {
         Self::Method(func.into())
     }
     /// Create a new [`CallableFunction::Plugin`].
     #[inline(always)]
+    #[must_use]
     pub fn from_plugin(func: impl PluginFunction + 'static + SendSync) -> Self {
         Self::Plugin((Box::new(func) as Box<FnPlugin>).into())
     }
