@@ -70,28 +70,30 @@ use std::prelude::v1::*;
 // Internal modules
 
 mod ast;
+mod custom_syntax;
 mod dynamic;
 mod engine;
 mod engine_api;
 mod engine_settings;
+mod error;
+mod error_parsing;
 mod fn_args;
 mod fn_builtin;
 mod fn_call;
 mod fn_func;
+mod fn_hash;
 mod fn_native;
+mod fn_ptr;
 mod fn_register;
+mod immutable_string;
 mod module;
 mod optimize;
 pub mod packages;
-mod parse_error;
-mod parser;
+mod parse;
 pub mod plugin;
-mod result;
 mod scope;
-mod syntax;
 mod token;
 mod r#unsafe;
-mod utils;
 
 type RhaiResult = Result<Dynamic, Box<EvalAltResult>>;
 
@@ -126,50 +128,53 @@ pub type FLOAT = f64;
 pub type FLOAT = f32;
 
 pub use ast::{FnAccess, AST};
+pub use custom_syntax::Expression;
 pub use dynamic::Dynamic;
 pub use engine::{Engine, EvalContext, OP_CONTAINS, OP_EQUALS};
-pub use fn_native::{FnPtr, NativeCallContext};
+pub use error::EvalAltResult;
+pub use error_parsing::{LexError, ParseError, ParseErrorType};
+pub use fn_native::NativeCallContext;
+pub use fn_ptr::FnPtr;
 pub use fn_register::RegisterNativeFunction;
+pub use immutable_string::ImmutableString;
 pub use module::{FnNamespace, Module};
-pub use parse_error::{LexError, ParseError, ParseErrorType};
-pub use result::EvalAltResult;
 pub use scope::Scope;
-pub use syntax::Expression;
 pub use token::Position;
-pub use utils::ImmutableString;
 
 /// An identifier in Rhai. [`SmartString`](https://crates.io/crates/smartstring) is used because most
 /// identifiers are ASCII and short, fewer than 23 characters, so they can be stored inline.
+#[cfg(not(feature = "internals"))]
 #[cfg(not(feature = "no_smartstring"))]
+pub(crate) type Identifier = SmartString;
+
+/// An identifier in Rhai.
+#[cfg(not(feature = "internals"))]
+#[cfg(feature = "no_smartstring")]
+pub(crate) type Identifier = ImmutableString;
+
+/// An identifier in Rhai. [`SmartString`](https://crates.io/crates/smartstring) is used because most
+/// identifiers are ASCII and short, fewer than 23 characters, so they can be stored inline.
+#[cfg(feature = "internals")]
+#[cfg(not(feature = "no_smartstring"))]
+#[deprecated = "this type is volatile and may change"]
 pub type Identifier = SmartString;
 
 /// An identifier in Rhai.
+#[cfg(feature = "internals")]
 #[cfg(feature = "no_smartstring")]
+#[deprecated = "this type is volatile and may change"]
 pub type Identifier = ImmutableString;
 
-/// A trait to enable registering Rust functions.
-/// This trait is  no longer needed and will be removed in the future.
-#[deprecated(
-    since = "0.19.15",
-    note = "this trait is no longer needed and will be removed in the future"
-)]
-pub trait RegisterFn {}
-
-/// A trait to enable registering Rust functions.
-/// This trait is  no longer needed and will be removed in the future.
-#[deprecated(
-    since = "0.19.15",
-    note = "this trait is no longer needed and will be removed in the future"
-)]
-pub trait RegisterResultFn {}
-
 /// Alias to [`Rc`][std::rc::Rc] or [`Arc`][std::sync::Arc] depending on the `sync` feature flag.
 pub use fn_native::Shared;
 
 #[cfg(not(feature = "no_closure"))]
 use fn_native::Locked;
 
-pub(crate) use utils::{calc_fn_hash, calc_fn_params_hash, calc_qualified_fn_hash, combine_hashes};
+pub(crate) use fn_hash::{
+    calc_fn_hash, calc_fn_params_hash, calc_qualified_fn_hash, calc_qualified_var_hash,
+    combine_hashes,
+};
 
 pub use rhai_codegen::*;
 
@@ -187,7 +192,7 @@ pub use ast::ScriptFnMetadata;
 #[cfg(not(feature = "no_index"))]
 pub type Array = Vec<Dynamic>;
 
-/// Hash map of [`Dynamic`] values with [`ImmutableString`] keys.
+/// Hash map of [`Dynamic`] values with [`SmartString`](https://crates.io/crates/smartstring) keys.
 /// Not available under `no_object`.
 #[cfg(not(feature = "no_object"))]
 pub type Map = std::collections::BTreeMap<Identifier, Dynamic>;
@@ -228,10 +233,11 @@ pub use ast::{
 
 #[cfg(feature = "internals")]
 #[deprecated = "this type is volatile and may change"]
-pub use engine::{Imports, State as EvalState};
+pub use engine::{EvalState, FnResolutionCache, FnResolutionCacheEntry, Imports};
 
 #[cfg(feature = "internals")]
 #[cfg(not(feature = "unchecked"))]
+#[deprecated = "this type is volatile and may change"]
 pub use engine::Limits;
 
 #[cfg(feature = "internals")]
@@ -307,17 +313,12 @@ type StaticVec<T> = smallvec::SmallVec<[T; 4]>;
 #[cfg(feature = "internals")]
 pub type StaticVec<T> = smallvec::SmallVec<[T; 4]>;
 
-#[cfg(not(feature = "internals"))]
 #[cfg(not(feature = "no_smartstring"))]
 pub(crate) type SmartString = smartstring::SmartString<smartstring::Compact>;
 
 #[cfg(feature = "no_smartstring")]
 pub(crate) type SmartString = String;
 
-#[cfg(feature = "internals")]
-#[cfg(not(feature = "no_smartstring"))]
-pub type SmartString = smartstring::SmartString<smartstring::Compact>;
-
 // Compiler guards against mutually-exclusive feature flags
 
 #[cfg(feature = "no_float")]
