@@ -2,10 +2,11 @@
 
 use crate::ast::{FnAccess, Ident};
 use crate::dynamic::Variant;
-use crate::fn_native::{shared_take_or_clone, CallableFunction, FnCallArgs, IteratorFn, SendSync};
+use crate::fn_call::FnCallArgs;
+use crate::fn_native::{shared_take_or_clone, CallableFunction, IteratorFn, SendSync};
 use crate::fn_register::RegisterNativeFunction;
+use crate::parse::IdentifierBuilder;
 use crate::token::Token;
-use crate::utils::IdentifierBuilder;
 use crate::{
     calc_fn_params_hash, calc_qualified_fn_hash, combine_hashes, Dynamic, EvalAltResult,
     Identifier, ImmutableString, NativeCallContext, Position, Shared, StaticVec,
@@ -67,11 +68,12 @@ impl FuncInfo {
     /// Generate a signature of the function.
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
+    #[must_use]
     pub fn gen_signature(&self) -> String {
         let mut sig = format!("{}(", self.name);
 
         if !self.param_names.is_empty() {
-            let mut params: Vec<String> =
+            let mut params: StaticVec<String> =
                 self.param_names.iter().map(|s| s.as_str().into()).collect();
             let return_type = params.pop().unwrap_or_else(|| "()".into());
             sig.push_str(&params.join(", "));
@@ -233,13 +235,13 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var_value::<i64>("answer").unwrap(), 42);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn new() -> Self {
         Self {
             id: None,
@@ -262,30 +264,21 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
-    /// module.set_id(Some("hello"));
+    /// module.set_id("hello");
     /// assert_eq!(module.id(), Some("hello"));
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn id(&self) -> Option<&str> {
         self.id_raw().map(|s| s.as_str())
     }
 
     /// Get the ID of the [`Module`] as an [`Identifier`], if any.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use rhai::Module;
-    ///
-    /// let mut module = Module::new();
-    /// module.set_id(Some("hello"));
-    /// assert_eq!(module.id_raw().map(|s| s.as_str()), Some("hello"));
-    /// ```
     #[inline(always)]
-    pub fn id_raw(&self) -> Option<&Identifier> {
+    #[must_use]
+    pub(crate) const fn id_raw(&self) -> Option<&Identifier> {
         self.id.as_ref()
     }
 
@@ -294,15 +287,31 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
-    /// module.set_id(Some("hello"));
+    /// module.set_id("hello");
     /// assert_eq!(module.id(), Some("hello"));
     /// ```
     #[inline(always)]
-    pub fn set_id<S: Into<Identifier>>(&mut self, id: Option<S>) -> &mut Self {
-        self.id = id.map(|s| s.into());
+    pub fn set_id(&mut self, id: impl Into<Identifier>) -> &mut Self {
+        self.id = Some(id.into());
+        self
+    }
+    /// Clear the ID of the [`Module`].
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use rhai::Module;
+    /// let mut module = Module::new();
+    /// module.set_id("hello");
+    /// assert_eq!(module.id(), Some("hello"));
+    /// module.clear_id();
+    /// assert_eq!(module.id(), None);
+    /// ```
+    #[inline(always)]
+    pub fn clear_id(&mut self) -> &mut Self {
+        self.id = None;
         self
     }
 
@@ -311,12 +320,12 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let module = Module::new();
     /// assert!(module.is_empty());
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn is_empty(&self) -> bool {
         self.functions.is_empty()
             && self.all_functions.is_empty()
@@ -334,8 +343,7 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// assert!(module.is_indexed());
     ///
@@ -349,7 +357,8 @@ impl Module {
     /// # }
     /// ```
     #[inline(always)]
-    pub fn is_indexed(&self) -> bool {
+    #[must_use]
+    pub const fn is_indexed(&self) -> bool {
         self.indexed
     }
 
@@ -357,6 +366,7 @@ impl Module {
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     #[inline(always)]
+    #[must_use]
     pub fn gen_fn_signatures(&self) -> impl Iterator<Item = String> + '_ {
         self.functions
             .values()
@@ -372,13 +382,13 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// module.set_var("answer", 42_i64);
     /// assert!(module.contains_var("answer"));
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn contains_var(&self, name: &str) -> bool {
         self.variables.contains_key(name)
     }
@@ -388,13 +398,13 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var_value::<i64>("answer").unwrap(), 42);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn get_var_value<T: Variant + Clone>(&self, name: &str) -> Option<T> {
         self.get_var(name).and_then(Dynamic::try_cast::<T>)
     }
@@ -404,13 +414,13 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var("answer").unwrap().cast::<i64>(), 42);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn get_var(&self, name: &str) -> Option<Dynamic> {
         self.variables.get(name).cloned()
     }
@@ -422,8 +432,7 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// module.set_var("answer", 42_i64);
     /// assert_eq!(module.get_var_value::<i64>("answer").unwrap(), 42);
@@ -438,7 +447,7 @@ impl Module {
         let value = Dynamic::from(value);
 
         if self.indexed {
-            let hash_var = crate::calc_qualified_fn_hash(once(""), &ident, 0);
+            let hash_var = crate::calc_qualified_var_hash(once(""), &ident);
             self.all_variables.insert(hash_var, value.clone());
         }
         self.variables.insert(ident, value);
@@ -448,6 +457,7 @@ impl Module {
     /// Get a reference to a namespace-qualified variable.
     /// Name and Position in [`EvalAltResult`] are [`None`] and [`NONE`][Position::NONE] and must be set afterwards.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_qualified_var(&self, hash_var: u64) -> Result<&Dynamic, Box<EvalAltResult>> {
         self.all_variables.get(&hash_var).ok_or_else(|| {
             EvalAltResult::ErrorVariableNotFound(String::new(), Position::NONE).into()
@@ -469,7 +479,7 @@ impl Module {
         param_names.push("Dynamic".into());
         self.functions.insert(
             hash_script,
-            Box::new(FuncInfo {
+            FuncInfo {
                 name: fn_def.name.clone(),
                 namespace: FnNamespace::Internal,
                 access: fn_def.access,
@@ -478,7 +488,8 @@ impl Module {
                 #[cfg(feature = "metadata")]
                 param_names,
                 func: Into::<CallableFunction>::into(fn_def).into(),
-            }),
+            }
+            .into(),
         );
         self.indexed = false;
         self.contains_indexed_global_functions = false;
@@ -489,6 +500,7 @@ impl Module {
     /// and number of parameters.
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub fn get_script_fn(
         &self,
         name: &str,
@@ -497,7 +509,7 @@ impl Module {
         self.functions
             .values()
             .find(|f| f.params == num_params && f.name == name)
-            .map(|f| f.func.get_fn_def())
+            .and_then(|f| f.func.get_script_fn_def())
     }
 
     /// Get a mutable reference to the underlying [`BTreeMap`] of sub-modules.
@@ -508,6 +520,7 @@ impl Module {
     /// Thus the [`Module`] is automatically set to be non-indexed.
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn sub_modules_mut(&mut self) -> &mut BTreeMap<Identifier, Shared<Module>> {
         // We must assume that the user has changed the sub-modules
         // (otherwise why take a mutable reference?)
@@ -525,14 +538,14 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let sub_module = Module::new();
     /// module.set_sub_module("question", sub_module);
     /// assert!(module.contains_sub_module("question"));
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn contains_sub_module(&self, name: &str) -> bool {
         self.modules.contains_key(name)
     }
@@ -542,14 +555,14 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let sub_module = Module::new();
     /// module.set_sub_module("question", sub_module);
     /// assert!(module.get_sub_module("question").is_some());
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn get_sub_module(&self, name: &str) -> Option<&Module> {
         self.modules.get(name).map(|m| m.as_ref())
     }
@@ -561,8 +574,7 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let sub_module = Module::new();
     /// module.set_sub_module("question", sub_module);
@@ -587,13 +599,13 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let hash = module.set_native_fn("calc", || Ok(42_i64));
     /// assert!(module.contains_fn(hash));
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn contains_fn(&self, hash_fn: u64) -> bool {
         self.functions.contains_key(&hash_fn)
     }
@@ -641,6 +653,7 @@ impl Module {
 
     /// Remap type ID.
     #[inline(always)]
+    #[must_use]
     fn map_type(map: bool, type_id: TypeId) -> TypeId {
         if !map {
             return type_id;
@@ -697,7 +710,7 @@ impl Module {
 
         self.functions.insert(
             hash_fn,
-            Box::new(FuncInfo {
+            FuncInfo {
                 name: self.identifiers.get(name),
                 namespace,
                 access,
@@ -706,7 +719,8 @@ impl Module {
                 #[cfg(feature = "metadata")]
                 param_names,
                 func: func.into(),
-            }),
+            }
+            .into(),
         );
 
         self.indexed = false;
@@ -827,8 +841,7 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let hash = module.set_native_fn("calc", || Ok(42_i64));
     /// assert!(module.contains_fn(hash));
@@ -862,8 +875,7 @@ impl Module {
     /// # Example
     ///
     /// ```
-    /// use rhai::Module;
-    ///
+    /// # use rhai::Module;
     /// let mut module = Module::new();
     /// let hash = module.set_getter_fn("value", |x: &mut i64| { Ok(*x) });
     /// assert!(module.contains_fn(hash));
@@ -1104,6 +1116,7 @@ impl Module {
     ///
     /// The [`u64`] hash is returned by the [`set_native_fn`][Module::set_native_fn] call.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_fn(&self, hash_fn: u64) -> Option<&CallableFunction> {
         self.functions.get(&hash_fn).map(|f| f.func.as_ref())
     }
@@ -1112,6 +1125,7 @@ impl Module {
     ///
     /// The [`u64`] hash is calculated by [`build_index`][Module::build_index].
     #[inline(always)]
+    #[must_use]
     pub fn contains_qualified_fn(&self, hash_fn: u64) -> bool {
         self.all_functions.contains_key(&hash_fn)
     }
@@ -1120,6 +1134,7 @@ impl Module {
     ///
     /// The [`u64`] hash is calculated by [`build_index`][Module::build_index].
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_qualified_fn(&self, hash_qualified_fn: u64) -> Option<&CallableFunction> {
         self.all_functions
             .get(&hash_qualified_fn)
@@ -1266,6 +1281,7 @@ impl Module {
 
     /// Get the number of variables, functions and type iterators in the [`Module`].
     #[inline(always)]
+    #[must_use]
     pub fn count(&self) -> (usize, usize, usize) {
         (
             self.variables.len(),
@@ -1276,12 +1292,14 @@ impl Module {
 
     /// Get an iterator to the sub-modules in the [`Module`].
     #[inline(always)]
+    #[must_use]
     pub fn iter_sub_modules(&self) -> impl Iterator<Item = (&str, Shared<Module>)> {
         self.modules.iter().map(|(k, m)| (k.as_str(), m.clone()))
     }
 
     /// Get an iterator to the variables in the [`Module`].
     #[inline(always)]
+    #[must_use]
     pub fn iter_var(&self) -> impl Iterator<Item = (&str, &Dynamic)> {
         self.variables.iter().map(|(k, v)| (k.as_str(), v))
     }
@@ -1289,6 +1307,7 @@ impl Module {
     /// Get an iterator to the functions in the [`Module`].
     #[inline(always)]
     #[allow(dead_code)]
+    #[must_use]
     pub(crate) fn iter_fn(&self) -> impl Iterator<Item = &FuncInfo> {
         self.functions.values().map(Box::as_ref)
     }
@@ -1303,6 +1322,7 @@ impl Module {
     /// 5) Shared reference to function definition [`ScriptFnDef`][crate::ast::ScriptFnDef].
     #[cfg(not(feature = "no_function"))]
     #[inline(always)]
+    #[must_use]
     pub(crate) fn iter_script_fn(
         &self,
     ) -> impl Iterator<
@@ -1323,7 +1343,9 @@ impl Module {
                     f.access,
                     f.name.as_str(),
                     f.params,
-                    f.func.get_fn_def(),
+                    f.func
+                        .get_script_fn_def()
+                        .expect("never fails because the function is scripted"),
                 )
             })
     }
@@ -1338,6 +1360,7 @@ impl Module {
     #[cfg(not(feature = "no_function"))]
     #[cfg(not(feature = "internals"))]
     #[inline(always)]
+    #[must_use]
     pub fn iter_script_fn_info(
         &self,
     ) -> impl Iterator<Item = (FnNamespace, FnAccess, &str, usize)> {
@@ -1359,6 +1382,7 @@ impl Module {
     #[cfg(not(feature = "no_function"))]
     #[cfg(feature = "internals")]
     #[inline(always)]
+    #[must_use]
     pub fn iter_script_fn_info(
         &self,
     ) -> impl Iterator<
@@ -1395,6 +1419,7 @@ impl Module {
     /// # }
     /// ```
     #[cfg(not(feature = "no_module"))]
+    #[must_use]
     pub fn eval_ast_as_new(
         mut scope: crate::Scope,
         ast: &crate::AST,
@@ -1445,13 +1470,23 @@ impl Module {
             .filter(|f| f.func.is_script())
             .for_each(|f| {
                 // Encapsulate AST environment
-                let mut func = f.func.get_fn_def().as_ref().clone();
+                let mut func = f
+                    .func
+                    .get_script_fn_def()
+                    .expect("never fails because the function is scripted")
+                    .as_ref()
+                    .clone();
                 func.lib = Some(ast.shared_lib());
                 func.mods = func_mods.clone();
                 module.set_script_fn(func);
             });
 
-        module.set_id(ast.clone_source());
+        if let Some(s) = ast.source_raw() {
+            module.set_id(s.clone());
+        } else {
+            module.clear_id();
+        }
+
         module.build_index();
 
         Ok(module)
@@ -1463,6 +1498,7 @@ impl Module {
     ///
     /// Panics if the [`Module`] is not yet indexed via [`build_index`][Module::build_index].
     #[inline(always)]
+    #[must_use]
     pub fn contains_indexed_global_functions(&self) -> bool {
         self.contains_indexed_global_functions
     }
@@ -1493,7 +1529,7 @@ impl Module {
 
             // Index all variables
             module.variables.iter().for_each(|(var_name, value)| {
-                let hash_var = crate::calc_qualified_fn_hash(path.iter().map(|&v| v), var_name, 0);
+                let hash_var = crate::calc_qualified_var_hash(path.iter().map(|&v| v), var_name);
                 variables.insert(hash_var, value.clone());
             });
 
@@ -1562,12 +1598,14 @@ impl Module {
 
     /// Does a type iterator exist in the entire module tree?
     #[inline(always)]
+    #[must_use]
     pub fn contains_qualified_iter(&self, id: TypeId) -> bool {
         self.all_type_iterators.contains_key(&id)
     }
 
     /// Does a type iterator exist in the module?
     #[inline(always)]
+    #[must_use]
     pub fn contains_iter(&self, id: TypeId) -> bool {
         self.type_iterators.contains_key(&id)
     }
@@ -1609,12 +1647,14 @@ impl Module {
 
     /// Get the specified type iterator.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_qualified_iter(&self, id: TypeId) -> Option<IteratorFn> {
         self.all_type_iterators.get(&id).cloned()
     }
 
     /// Get the specified type iterator.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_iter(&self, id: TypeId) -> Option<IteratorFn> {
         self.type_iterators.get(&id).cloned()
     }
@@ -1649,7 +1689,7 @@ impl fmt::Debug for NamespaceRef {
                 .path
                 .iter()
                 .map(|Ident { name, .. }| name.as_str())
-                .collect::<Vec<_>>()
+                .collect::<StaticVec<_>>()
                 .join("::"),
         )
     }
@@ -1691,9 +1731,16 @@ impl From<StaticVec<Ident>> for NamespaceRef {
 }
 
 impl NamespaceRef {
+    /// Create a new [`NamespaceRef`].
+    #[inline(always)]
+    #[must_use]
+    pub fn new(&self) -> Self {
+        Default::default()
+    }
     /// Get the [`Scope`][crate::Scope] index offset.
     #[inline(always)]
-    pub(crate) fn index(&self) -> Option<NonZeroUsize> {
+    #[must_use]
+    pub(crate) const fn index(&self) -> Option<NonZeroUsize> {
         self.index
     }
     /// Set the [`Scope`][crate::Scope] index offset.
