@@ -11,7 +11,7 @@ def_package!(crate:BasicFnPackage:"Basic Fn functions.", lib, {
 mod fn_ptr_functions {
     #[rhai_fn(name = "name", get = "name", pure)]
     pub fn name(f: &mut FnPtr) -> ImmutableString {
-        f.get_fn_name().as_str().into()
+        f.fn_name_raw().into()
     }
 
     #[cfg(not(feature = "no_function"))]
