@@ -10,7 +10,7 @@ use num_traits::{CheckedAdd as Add, CheckedSub as Sub};
 #[cfg(feature = "unchecked")]
 use std::ops::{Add, Sub};
 
-// Register range function with step
+// Range iterator with step
 #[derive(Debug, Clone, Copy, Hash, Eq, PartialEq)]
 struct StepRange<T>(T, T, T)
 where
@@ -27,10 +27,11 @@ where
                 return EvalAltResult::ErrorInFunctionCall(
                     "range".to_string(),
                     Default::default(),
-                    Box::new(EvalAltResult::ErrorArithmetic(
+                    EvalAltResult::ErrorArithmetic(
                         "step value cannot be zero".to_string(),
                         crate::Position::NONE,
-                    )),
+                    )
+                    .into(),
                     crate::Position::NONE,
                 )
                 .into();
@@ -126,6 +127,129 @@ where
     }
 }
 
+// Bit-field iterator with step
+#[derive(Debug, Clone, Copy, Hash, Eq, PartialEq)]
+struct BitRange(INT, INT, usize);
+
+const BITS: usize = std::mem::size_of::<INT>() * 8;
+
+impl BitRange {
+    pub fn new(value: INT, from: INT, len: INT) -> Result<Self, Box<EvalAltResult>> {
+        let from = if from >= 0 {
+            let offset = from as usize;
+
+            #[cfg(not(feature = "unchecked"))]
+            if offset >= BITS {
+                return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
+                    .into();
+            }
+            offset
+        } else {
+            #[cfg(not(feature = "unchecked"))]
+            if let Some(abs_from) = from.checked_abs() {
+                if (abs_from as usize) > BITS {
+                    return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
+                        .into();
+                }
+                BITS - (abs_from as usize)
+            } else {
+                return EvalAltResult::ErrorBitFieldBounds(BITS, from, crate::Position::NONE)
+                    .into();
+            }
+
+            #[cfg(feature = "unchecked")]
+            {
+                BITS - (from.abs() as usize)
+            }
+        };
+
+        let len = if len < 0 {
+            0
+        } else if from + (len as usize) > BITS {
+            BITS - from
+        } else {
+            len as usize
+        };
+
+        Ok(Self(value, 1 << from, len))
+    }
+}
+
+impl Iterator for BitRange {
+    type Item = bool;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let Self(value, mask, len) = *self;
+
+        if len == 0 {
+            None
+        } else {
+            let r = (value & mask) != 0;
+            self.1 <<= 1;
+            self.2 -= 1;
+            Some(r)
+        }
+    }
+}
+
+// String iterator over characters
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
+struct CharsStream(Vec<char>, usize);
+
+impl CharsStream {
+    pub fn new(string: &str, from: INT, len: INT) -> Self {
+        if len <= 0 {
+            return Self(Default::default(), 0);
+        }
+        if from >= 0 {
+            return Self(
+                string
+                    .chars()
+                    .skip(from as usize)
+                    .take(len as usize)
+                    .collect(),
+                0,
+            );
+        }
+        #[cfg(not(feature = "unchecked"))]
+        return if let Some(abs_from) = from.checked_abs() {
+            let num_chars = string.chars().count();
+            let offset = if num_chars < (abs_from as usize) {
+                0
+            } else {
+                num_chars - (abs_from as usize)
+            };
+            Self(string.chars().skip(offset).take(len as usize).collect(), 0)
+        } else {
+            Self(string.chars().skip(0).take(len as usize).collect(), 0)
+        };
+
+        #[cfg(feature = "unchecked")]
+        return Self(
+            string
+                .chars()
+                .skip(from as usize)
+                .take(len as usize)
+                .collect(),
+            0,
+        );
+    }
+}
+
+impl Iterator for CharsStream {
+    type Item = char;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        if self.1 >= self.0.len() {
+            None
+        } else {
+            let ch = self.0[self.1];
+            self.1 += 1;
+            Some(ch)
+        }
+    }
+}
+
 macro_rules! reg_range {
     ($lib:ident | $x:expr => $( $y:ty ),*) => {
         $(
@@ -164,9 +288,8 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
     {
         reg_range!(lib | "range" => i8, u8, i16, u16, i32, u32, i64, u64);
 
-        if cfg!(not(target_arch = "wasm32")) {
-            reg_range!(lib | "range" => i128, u128);
-        }
+        #[cfg(not(target_arch = "wasm32"))]
+        reg_range!(lib | "range" => i128, u128);
     }
 
     reg_range!(lib | step "range" => INT);
@@ -176,9 +299,8 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
     {
         reg_range!(lib | step "range" => i8, u8, i16, u16, i32, u32, i64, u64);
 
-        if cfg!(not(target_arch = "wasm32")) {
-            reg_range!(lib | step "range" => i128, u128);
-        }
+        #[cfg(not(target_arch = "wasm32"))]
+        reg_range!(lib | step "range" => i128, u128);
     }
 
     #[cfg(not(feature = "no_float"))]
@@ -193,7 +315,7 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
                 #[cfg(not(feature = "unchecked"))]
                 if step == 0.0 {
                     return EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
-                        Box::new(EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE)),
+                        EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE).into(),
                         crate::Position::NONE,
                     ).into();
                 }
@@ -255,7 +377,7 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
                 #[cfg(not(feature = "unchecked"))]
                 if step.is_zero() {
                     return EvalAltResult::ErrorInFunctionCall("range".to_string(), "".to_string(),
-                        Box::new(EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE)),
+                        EvalAltResult::ErrorArithmetic("step value cannot be zero".to_string(), crate::Position::NONE).into(),
                         crate::Position::NONE,
                     ).into();
                 }
@@ -304,4 +426,34 @@ def_package!(crate:BasicIteratorPackage:"Basic range iterators.", lib, {
         #[cfg(feature = "metadata")]
         lib.update_fn_metadata(_hash, &["from: Decimal", "to: Decimal", "step: Decimal", "Iterator<Item=Decimal>"]);
     }
+
+    // Register string iterator
+    lib.set_iterator::<CharsStream>();
+
+    let _hash = lib.set_native_fn("chars", |string, from,len| Ok(CharsStream::new(string, from, len)));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["string: &str", "from: INT", "len: INT", "Iterator<Item=char>"]);
+
+    let _hash = lib.set_native_fn("chars", |string, from| Ok(CharsStream::new(string, from, INT::MAX)));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["string: &str", "from: INT", "Iterator<Item=char>"]);
+
+    let _hash = lib.set_native_fn("chars", |string| Ok(CharsStream::new(string, 0, INT::MAX)));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["string: &str", "Iterator<Item=char>"]);
+
+    // Register bit-field iterator
+    lib.set_iterator::<BitRange>();
+
+    let _hash = lib.set_native_fn("bits", |value, from, len| BitRange::new(value, from, len));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["value: INT", "from: INT", "len: INT", "Iterator<Item=bool>"]);
+
+    let _hash = lib.set_native_fn("bits", |value, from| BitRange::new(value, from, INT::MAX));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["value: INT", "from: INT", "Iterator<Item=bool>"]);
+
+    let _hash = lib.set_native_fn("bits", |value| BitRange::new(value, 0, INT::MAX));
+    #[cfg(feature = "metadata")]
+    lib.update_fn_metadata(_hash, &["value: INT", "Iterator<Item=bool>"]);
 });
