@@ -1,6 +1,7 @@
 use super::arithmetic::ArithmeticPackage;
 use super::fn_basic::BasicFnPackage;
 use super::iter_basic::BasicIteratorPackage;
+use super::lang_core::LanguageCorePackage;
 use super::logic::LogicPackage;
 use super::string_basic::BasicStringPackage;
 #[cfg(feature = "no_std")]
@@ -9,6 +10,7 @@ use std::prelude::v1::*;
 use crate::def_package;
 
 def_package!(crate:CorePackage:"_Core_ package containing basic facilities.", lib, {
+    LanguageCorePackage::init(lib);
     ArithmeticPackage::init(lib);
     LogicPackage::init(lib);
     BasicStringPackage::init(lib);
