@@ -1,7 +1,7 @@
 #![allow(non_snake_case)]
 
 use crate::plugin::*;
-use crate::{def_package, Dynamic, ImmutableString, StaticVec, INT};
+use crate::{def_package, Dynamic, StaticVec, INT};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{any::TypeId, mem};
@@ -10,12 +10,6 @@ use super::string_basic::{print_with_func, FUNC_TO_STRING};
 
 def_package!(crate:MoreStringPackage:"Additional string utilities, including string building.", lib, {
     combine_with_exported_module!(lib, "string", string_functions);
-
-    // Register string iterator
-    lib.set_iter(
-        TypeId::of::<ImmutableString>(),
-        |string| Box::new(string.cast::<ImmutableString>().chars().collect::<Vec<_>>().into_iter().map(Into::into))
-    );
 });
 
 #[export_module]
@@ -106,11 +100,57 @@ mod string_functions {
         }
     }
 
+    pub fn to_upper(string: &str) -> ImmutableString {
+        string.to_uppercase().into()
+    }
+    pub fn make_upper(string: &mut ImmutableString) {
+        *string = to_upper(string);
+    }
+    pub fn to_lower(string: &str) -> ImmutableString {
+        string.to_lowercase().into()
+    }
+    pub fn make_lower(string: &mut ImmutableString) {
+        *string = to_lower(string);
+    }
+
+    #[rhai_fn(name = "to_upper")]
+    pub fn to_upper_char(character: char) -> char {
+        let mut stream = character.to_uppercase();
+        let ch = stream
+            .next()
+            .expect("never fails because there should be at least one character");
+        if stream.next().is_some() {
+            character
+        } else {
+            ch
+        }
+    }
+    #[rhai_fn(name = "make_upper")]
+    pub fn make_upper_char(character: &mut char) {
+        *character = to_upper_char(*character)
+    }
+    #[rhai_fn(name = "to_lower")]
+    pub fn to_lower_char(character: char) -> char {
+        let mut stream = character.to_lowercase();
+        let ch = stream
+            .next()
+            .expect("never fails because there should be at least one character");
+        if stream.next().is_some() {
+            character
+        } else {
+            ch
+        }
+    }
+    #[rhai_fn(name = "make_lower")]
+    pub fn make_lower_char(character: &mut char) {
+        *character = to_lower_char(*character)
+    }
+
     #[rhai_fn(name = "index_of")]
     pub fn index_of_char_starting_from(string: &str, character: char, start: INT) -> INT {
         let start = if start < 0 {
             if let Some(n) = start.checked_abs() {
-                let chars = string.chars().collect::<Vec<_>>();
+                let chars: Vec<_> = string.chars().collect();
                 let num_chars = chars.len();
                 if n as usize > num_chars {
                     0
