@@ -1,6 +1,7 @@
 //! Module defining macros for developing _plugins_.
 
-pub use crate::fn_native::{CallableFunction, FnCallArgs};
+use crate::fn_call::FnCallArgs;
+pub use crate::fn_native::CallableFunction;
 pub use crate::{
     Dynamic, Engine, EvalAltResult, FnAccess, FnNamespace, ImmutableString, Module,
     NativeCallContext, Position,
@@ -21,11 +22,10 @@ pub use rhai_codegen::{export_fn, register_exported_fn};
 /// Use the `#[export_module]` and `#[export_fn]` procedural attributes instead.
 pub trait PluginFunction {
     /// Call the plugin function with the arguments provided.
+    #[must_use]
     fn call(&self, context: NativeCallContext, args: &mut FnCallArgs) -> RhaiResult;
 
     /// Is this plugin function a method?
+    #[must_use]
     fn is_method_call(&self) -> bool;
-
-    /// Is this plugin function variadic?
-    fn is_variadic(&self) -> bool;
 }
