@@ -7,7 +7,7 @@ use std::prelude::v1::*;
 use std::{borrow::Cow, iter::Extend};
 
 /// Keep a number of entries inline (since [`Dynamic`] is usually small enough).
-const SCOPE_SIZE: usize = 8;
+const SCOPE_ENTRIES_INLINED: usize = 8;
 
 /// Type containing information about the current scope.
 /// Useful for keeping state between [`Engine`][crate::Engine] evaluation runs.
@@ -49,14 +49,14 @@ const SCOPE_SIZE: usize = 8;
 // look up a variable.  Variable lookup is usually via direct indexing, by-passing the name altogether.
 //
 // Since [`Dynamic`] is reasonably small, packing it tightly improves cache locality when variables are accessed.
-//
-// The alias is `Box`'ed because it occurs infrequently.
 #[derive(Debug, Clone, Hash)]
 pub struct Scope<'a> {
     /// Current value of the entry.
-    values: smallvec::SmallVec<[Dynamic; SCOPE_SIZE]>,
+    values: smallvec::SmallVec<[Dynamic; SCOPE_ENTRIES_INLINED]>,
     /// (Name, aliases) of the entry.
-    names: Vec<(Cow<'a, str>, Option<Box<StaticVec<Identifier>>>)>,
+    names: smallvec::SmallVec<
+        [(Cow<'a, str>, Option<Box<StaticVec<Identifier>>>); SCOPE_ENTRIES_INLINED],
+    >,
 }
 
 impl Default for Scope<'_> {
@@ -95,6 +95,7 @@ impl<'a> Scope<'a> {
     /// assert_eq!(my_scope.get_value::<i64>("x").unwrap(), 42);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn new() -> Self {
         Self {
             values: Default::default(),
@@ -140,6 +141,7 @@ impl<'a> Scope<'a> {
     /// assert_eq!(my_scope.len(), 1);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn len(&self) -> usize {
         self.values.len()
     }
@@ -157,8 +159,9 @@ impl<'a> Scope<'a> {
     /// assert!(!my_scope.is_empty());
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn is_empty(&self) -> bool {
-        self.values.len() == 0
+        self.values.is_empty()
     }
     /// Add (push) a new entry to the [`Scope`].
     ///
@@ -250,7 +253,7 @@ impl<'a> Scope<'a> {
         access: AccessMode,
         mut value: Dynamic,
     ) -> &mut Self {
-        self.names.push((name.into(), None));
+        self.names.push((name.into(), Default::default()));
         value.set_access_mode(access);
         self.values.push(value.into());
         self
@@ -301,6 +304,7 @@ impl<'a> Scope<'a> {
     /// assert!(!my_scope.contains("y"));
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn contains(&self, name: &str) -> bool {
         self.names
             .iter()
@@ -309,6 +313,7 @@ impl<'a> Scope<'a> {
     }
     /// Find an entry in the [`Scope`], starting from the last.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_index(&self, name: &str) -> Option<(usize, AccessMode)> {
         self.names
             .iter()
@@ -335,6 +340,7 @@ impl<'a> Scope<'a> {
     /// assert_eq!(my_scope.get_value::<i64>("x").unwrap(), 42);
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn get_value<T: Variant + Clone>(&self, name: &str) -> Option<T> {
         self.names
             .iter()
@@ -402,6 +408,7 @@ impl<'a> Scope<'a> {
     ///
     /// assert_eq!(my_scope.get_value::<i64>("x").unwrap(), 123);
     /// ```
+    #[must_use]
     pub fn get_mut(&mut self, name: &str) -> Option<&mut Dynamic> {
         self.get_index(name)
             .and_then(move |(index, access)| match access {
@@ -415,6 +422,7 @@ impl<'a> Scope<'a> {
     ///
     /// Panics if the index is out of bounds.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn get_mut_by_index(&mut self, index: usize) -> &mut Dynamic {
         self.values
             .get_mut(index)
@@ -428,26 +436,25 @@ impl<'a> Scope<'a> {
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
     pub(crate) fn add_entry_alias(&mut self, index: usize, alias: Identifier) -> &mut Self {
-        let entry = self
+        let (_, aliases) = self
             .names
             .get_mut(index)
             .expect("never fails unless the index is out of bounds");
-        if entry.1.is_none() {
-            // Initialize the alias list if it is empty.
-            entry.1 = Some(Default::default());
-        }
-        let list = entry
-            .1
-            .as_mut()
-            .expect("never fails because the list is initialized");
-        if !list.iter().any(|a| &alias == a) {
-            list.push(alias);
+        match aliases {
+            None => {
+                let mut list = StaticVec::new();
+                list.push(alias);
+                *aliases = Some(list.into());
+            }
+            Some(aliases) if !aliases.iter().any(|a| a == &alias) => aliases.push(alias),
+            Some(_) => (),
         }
         self
     }
     /// Clone the [`Scope`], keeping only the last instances of each variable name.
     /// Shadowed variables are omitted in the copy.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn clone_visible(&self) -> Self {
         let mut entries: Self = Default::default();
 
@@ -467,6 +474,7 @@ impl<'a> Scope<'a> {
     /// Get an iterator to entries in the [`Scope`].
     #[inline(always)]
     #[allow(dead_code)]
+    #[must_use]
     pub(crate) fn into_iter(
         self,
     ) -> impl Iterator<Item = (Cow<'a, str>, Dynamic, Vec<Identifier>)> {
@@ -503,6 +511,7 @@ impl<'a> Scope<'a> {
     /// assert_eq!(value.cast::<String>(), "hello");
     /// ```
     #[inline(always)]
+    #[must_use]
     pub fn iter(&self) -> impl Iterator<Item = (&str, bool, Dynamic)> {
         self.iter_raw()
             .map(|(name, constant, value)| (name, constant, value.flatten_clone()))
@@ -510,6 +519,7 @@ impl<'a> Scope<'a> {
     /// Get an iterator to entries in the [`Scope`].
     /// Shared values are not expanded.
     #[inline(always)]
+    #[must_use]
     pub fn iter_raw(&self) -> impl Iterator<Item = (&str, bool, &Dynamic)> {
         self.names
             .iter()
@@ -522,7 +532,7 @@ impl<'a, K: Into<Cow<'a, str>>> Extend<(K, Dynamic)> for Scope<'a> {
     #[inline(always)]
     fn extend<T: IntoIterator<Item = (K, Dynamic)>>(&mut self, iter: T) {
         iter.into_iter().for_each(|(name, value)| {
-            self.names.push((name.into(), None));
+            self.names.push((name.into(), Default::default()));
             self.values.push(value);
         });
     }
