@@ -28,14 +28,17 @@ impl<'de> DynamicDeserializer<'de> {
     ///
     /// The reference is necessary because the deserialized type may hold references
     /// (especially `&str`) to the source [`Dynamic`][crate::Dynamic].
+    #[must_use]
     pub fn from_dynamic(value: &'de Dynamic) -> Self {
         Self { value }
     }
     /// Shortcut for a type conversion error.
+    #[must_use]
     fn type_error<T>(&self) -> Result<T, Box<EvalAltResult>> {
         self.type_error_str(type_name::<T>())
     }
     /// Shortcut for a type conversion error.
+    #[must_use]
     fn type_error_str<T>(&self, error: &str) -> Result<T, Box<EvalAltResult>> {
         EvalAltResult::ErrorMismatchOutputType(
             error.into(),
@@ -44,6 +47,7 @@ impl<'de> DynamicDeserializer<'de> {
         )
         .into()
     }
+    #[must_use]
     fn deserialize_int<V: Visitor<'de>>(
         &mut self,
         v: crate::INT,
@@ -107,6 +111,7 @@ impl<'de> DynamicDeserializer<'de> {
 /// # Ok(())
 /// # }
 /// ```
+#[must_use]
 pub fn from_dynamic<'de, T: Deserialize<'de>>(
     value: &'de Dynamic,
 ) -> Result<T, Box<EvalAltResult>> {
@@ -125,7 +130,7 @@ impl<'de> Deserializer<'de> for &mut DynamicDeserializer<'de> {
     type Error = Box<EvalAltResult>;
 
     fn deserialize_any<V: Visitor<'de>>(self, visitor: V) -> Result<V::Value, Box<EvalAltResult>> {
-        match &self.value.0 {
+        match self.value.0 {
             Union::Unit(_, _, _) => self.deserialize_unit(visitor),
             Union::Bool(_, _, _) => self.deserialize_bool(visitor),
             Union::Str(_, _, _) => self.deserialize_str(visitor),
@@ -158,16 +163,16 @@ impl<'de> Deserializer<'de> for &mut DynamicDeserializer<'de> {
             #[cfg(not(feature = "no_std"))]
             Union::TimeStamp(_, _, _) => self.type_error(),
 
-            Union::Variant(value, _, _) if value.is::<i8>() => self.deserialize_i8(visitor),
-            Union::Variant(value, _, _) if value.is::<i16>() => self.deserialize_i16(visitor),
-            Union::Variant(value, _, _) if value.is::<i32>() => self.deserialize_i32(visitor),
-            Union::Variant(value, _, _) if value.is::<i64>() => self.deserialize_i64(visitor),
-            Union::Variant(value, _, _) if value.is::<i128>() => self.deserialize_i128(visitor),
-            Union::Variant(value, _, _) if value.is::<u8>() => self.deserialize_u8(visitor),
-            Union::Variant(value, _, _) if value.is::<u16>() => self.deserialize_u16(visitor),
-            Union::Variant(value, _, _) if value.is::<u32>() => self.deserialize_u32(visitor),
-            Union::Variant(value, _, _) if value.is::<u64>() => self.deserialize_u64(visitor),
-            Union::Variant(value, _, _) if value.is::<u128>() => self.deserialize_u128(visitor),
+            Union::Variant(ref value, _, _) if value.is::<i8>() => self.deserialize_i8(visitor),
+            Union::Variant(ref value, _, _) if value.is::<i16>() => self.deserialize_i16(visitor),
+            Union::Variant(ref value, _, _) if value.is::<i32>() => self.deserialize_i32(visitor),
+            Union::Variant(ref value, _, _) if value.is::<i64>() => self.deserialize_i64(visitor),
+            Union::Variant(ref value, _, _) if value.is::<i128>() => self.deserialize_i128(visitor),
+            Union::Variant(ref value, _, _) if value.is::<u8>() => self.deserialize_u8(visitor),
+            Union::Variant(ref value, _, _) if value.is::<u16>() => self.deserialize_u16(visitor),
+            Union::Variant(ref value, _, _) if value.is::<u32>() => self.deserialize_u32(visitor),
+            Union::Variant(ref value, _, _) if value.is::<u64>() => self.deserialize_u64(visitor),
+            Union::Variant(ref value, _, _) if value.is::<u128>() => self.deserialize_u128(visitor),
 
             Union::Variant(_, _, _) => self.type_error(),
 
@@ -494,6 +499,7 @@ struct IterateArray<'a, ITER: Iterator<Item = &'a Dynamic>> {
 
 #[cfg(not(feature = "no_index"))]
 impl<'a, ITER: Iterator<Item = &'a Dynamic>> IterateArray<'a, ITER> {
+    #[must_use]
     pub fn new(iter: ITER) -> Self {
         Self { iter }
     }
@@ -534,6 +540,7 @@ where
     KEYS: Iterator<Item = &'a str>,
     VALUES: Iterator<Item = &'a Dynamic>,
 {
+    #[must_use]
     pub fn new(keys: KEYS, values: VALUES) -> Self {
         Self { keys, values }
     }
