@@ -146,7 +146,11 @@ impl From<&crate::module::FuncInfo> for FnMetadata {
                 }
                 #[cfg(not(feature = "no_function"))]
                 {
-                    info.func.get_fn_def().comments.to_vec()
+                    info.func
+                        .get_script_fn_def()
+                        .expect("never fails because the function is scripted")
+                        .comments
+                        .to_vec()
                 }
             } else {
                 Default::default()
@@ -214,6 +218,7 @@ impl Engine {
     /// 2) Functions registered into the global namespace
     /// 3) Functions in static modules
     /// 4) Functions in global modules (optional)
+    #[must_use]
     pub fn gen_fn_metadata_with_ast_to_json(
         &self,
         ast: &AST,
@@ -253,6 +258,7 @@ impl Engine {
     /// 1) Functions registered into the global namespace
     /// 2) Functions in static modules
     /// 3) Functions in global modules (optional)
+    #[must_use]
     pub fn gen_fn_metadata_to_json(&self, include_global: bool) -> serde_json::Result<String> {
         self.gen_fn_metadata_with_ast_to_json(&Default::default(), include_global)
     }
