@@ -25,6 +25,7 @@ struct DynamicSerializer {
 
 impl DynamicSerializer {
     /// Create a [`DynamicSerializer`] from a [`Dynamic`][crate::Dynamic] value.
+    #[must_use]
     pub fn new(_value: Dynamic) -> Self {
         Self {
             _key: Default::default(),
@@ -81,6 +82,7 @@ impl DynamicSerializer {
 /// # Ok(())
 /// # }
 /// ```
+#[must_use]
 pub fn to_dynamic<T: Serialize>(value: T) -> RhaiResult {
     let mut s = DynamicSerializer::new(Default::default());
     value.serialize(&mut s)
@@ -542,7 +544,7 @@ impl SerializeMap for DynamicSerializer {
         #[cfg(not(feature = "no_object"))]
         {
             let key = std::mem::take(&mut self._key)
-                .take_immutable_string()
+                .as_immutable_string()
                 .map_err(|typ| {
                     EvalAltResult::ErrorMismatchDataType(
                         "string".into(),
@@ -572,7 +574,7 @@ impl SerializeMap for DynamicSerializer {
         #[cfg(not(feature = "no_object"))]
         {
             let _key: Dynamic = _key.serialize(&mut *self)?;
-            let _key = _key.take_immutable_string().map_err(|typ| {
+            let _key = _key.as_immutable_string().map_err(|typ| {
                 EvalAltResult::ErrorMismatchDataType("string".into(), typ.into(), Position::NONE)
             })?;
             let _value = _value.serialize(&mut *self)?;
