@@ -41,7 +41,7 @@ pub type TokenizerControl = Rc<Cell<TokenizerControlBlock>>;
 type LERR = LexError;
 
 /// Separator character for numbers.
-const NUM_SEP: char = '_';
+const NUMBER_SEPARATOR: char = '_';
 
 /// A stream of tokens.
 pub type TokenStream<'a> = Peekable<TokenIterator<'a>>;
@@ -89,6 +89,7 @@ impl Position {
     ///
     /// Panics if `line` is zero.
     #[inline(always)]
+    #[must_use]
     pub fn new(line: u16, _position: u16) -> Self {
         assert!(line != 0, "line cannot be zero");
 
@@ -101,6 +102,7 @@ impl Position {
     }
     /// Get the line number (1-based), or [`None`] if there is no position.
     #[inline(always)]
+    #[must_use]
     pub fn line(self) -> Option<usize> {
         if self.is_none() {
             None
@@ -108,11 +110,12 @@ impl Position {
             #[cfg(not(feature = "no_position"))]
             return Some(self.line as usize);
             #[cfg(feature = "no_position")]
-            unreachable!();
+            unreachable!("there is no Position");
         }
     }
     /// Get the character position (1-based), or [`None`] if at beginning of a line.
     #[inline(always)]
+    #[must_use]
     pub fn position(self) -> Option<usize> {
         if self.is_none() {
             None
@@ -125,7 +128,7 @@ impl Position {
             };
 
             #[cfg(feature = "no_position")]
-            unreachable!();
+            unreachable!("there is no Position");
         }
     }
     /// Advance by one character position.
@@ -171,7 +174,8 @@ impl Position {
     }
     /// Is this [`Position`] at the beginning of a line?
     #[inline(always)]
-    pub fn is_beginning_of_line(self) -> bool {
+    #[must_use]
+    pub const fn is_beginning_of_line(self) -> bool {
         #[cfg(not(feature = "no_position"))]
         return self.pos == 0 && !self.is_none();
         #[cfg(feature = "no_position")]
@@ -179,14 +183,16 @@ impl Position {
     }
     /// Is there no [`Position`]?
     #[inline(always)]
-    pub fn is_none(self) -> bool {
+    #[must_use]
+    pub const fn is_none(self) -> bool {
         #[cfg(not(feature = "no_position"))]
-        return self == Self::NONE;
+        return self.line == 0 && self.pos == 0;
         #[cfg(feature = "no_position")]
         return true;
     }
     /// Print this [`Position`] for debug purposes.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn debug_print(&self, _f: &mut fmt::Formatter<'_>) -> fmt::Result {
         #[cfg(not(feature = "no_position"))]
         if !self.is_none() {
@@ -461,11 +467,8 @@ pub enum Token {
 
 impl Token {
     /// Get the syntax of the token if it is a keyword.
-    ///
-    /// # Panics
-    ///
-    /// Panics if the token is not a keyword.
-    pub fn keyword_syntax(&self) -> &'static str {
+    #[must_use]
+    pub const fn keyword_syntax(&self) -> &'static str {
         use Token::*;
 
         match self {
@@ -549,11 +552,12 @@ impl Token {
             #[cfg(not(feature = "no_module"))]
             As => "as",
 
-            t => unreachable!("{:?} is not a keyword", t),
+            _ => "ERROR: NOT A KEYWORD",
         }
     }
 
     /// Get the syntax of the token.
+    #[must_use]
     pub fn syntax(&self) -> Cow<'static, str> {
         use Token::*;
 
@@ -580,7 +584,8 @@ impl Token {
 
     /// Is this token an op-assignment operator?
     #[inline]
-    pub fn is_op_assignment(&self) -> bool {
+    #[must_use]
+    pub const fn is_op_assignment(&self) -> bool {
         match self {
             Self::PlusAssign
             | Self::MinusAssign
@@ -598,7 +603,8 @@ impl Token {
     }
 
     /// Get the corresponding operator of the token if it is an op-assignment operator.
-    pub fn map_op_assignment(&self) -> Option<Self> {
+    #[must_use]
+    pub const fn map_op_assignment(&self) -> Option<Self> {
         Some(match self {
             Self::PlusAssign => Self::Plus,
             Self::MinusAssign => Self::Minus,
@@ -617,7 +623,8 @@ impl Token {
 
     /// Has this token a corresponding op-assignment operator?
     #[inline]
-    pub fn has_op_assignment(&self) -> bool {
+    #[must_use]
+    pub const fn has_op_assignment(&self) -> bool {
         match self {
             Self::Plus
             | Self::Minus
@@ -635,7 +642,8 @@ impl Token {
     }
 
     /// Get the corresponding op-assignment operator of the token.
-    pub fn make_op_assignment(&self) -> Option<Self> {
+    #[must_use]
+    pub const fn make_op_assignment(&self) -> Option<Self> {
         Some(match self {
             Self::Plus => Self::PlusAssign,
             Self::Minus => Self::MinusAssign,
@@ -653,6 +661,7 @@ impl Token {
     }
 
     /// Reverse lookup a token from a piece of syntax.
+    #[must_use]
     pub fn lookup_from_syntax(syntax: &str) -> Option<Self> {
         use Token::*;
 
@@ -743,11 +752,9 @@ impl Token {
 
             "===" | "!==" | "->" | "<-" | ":=" | "~" | "::<" | "(*" | "*)" | "#" | "#!"
             | "public" | "protected" | "super" | "new" | "use" | "module" | "package" | "var"
-            | "static" | "begin" | "end" | "shared" | "with" | "each" | "then" | "goto"
-            | "unless" | "exit" | "match" | "case" | "default" | "void" | "null" | "nil"
-            | "spawn" | "thread" | "go" | "sync" | "async" | "await" | "yield" => {
-                Reserved(syntax.into())
-            }
+            | "static" | "shared" | "with" | "goto" | "exit" | "match" | "case" | "default"
+            | "void" | "null" | "nil" | "spawn" | "thread" | "go" | "sync" | "async" | "await"
+            | "yield" => Reserved(syntax.into()),
 
             KEYWORD_PRINT | KEYWORD_DEBUG | KEYWORD_TYPE_OF | KEYWORD_EVAL | KEYWORD_FN_PTR
             | KEYWORD_FN_PTR_CALL | KEYWORD_FN_PTR_CURRY | KEYWORD_THIS | KEYWORD_IS_DEF_VAR => {
@@ -763,7 +770,8 @@ impl Token {
 
     // Is this token [`EOF`][Token::EOF]?
     #[inline(always)]
-    pub fn is_eof(&self) -> bool {
+    #[must_use]
+    pub const fn is_eof(&self) -> bool {
         use Token::*;
 
         match self {
@@ -774,7 +782,8 @@ impl Token {
 
     // If another operator is after these, it's probably an unary operator
     // (not sure about `fn` name).
-    pub fn is_next_unary(&self) -> bool {
+    #[must_use]
+    pub const fn is_next_unary(&self) -> bool {
         use Token::*;
 
         match self {
@@ -834,7 +843,8 @@ impl Token {
     }
 
     /// Get the precedence number of the token.
-    pub fn precedence(&self) -> Option<Precedence> {
+    #[must_use]
+    pub const fn precedence(&self) -> Option<Precedence> {
         use Token::*;
 
         Precedence::new(match self {
@@ -868,7 +878,8 @@ impl Token {
     }
 
     /// Does an expression bind to the right (instead of left)?
-    pub fn is_bind_right(&self) -> bool {
+    #[must_use]
+    pub const fn is_bind_right(&self) -> bool {
         use Token::*;
 
         match self {
@@ -888,7 +899,8 @@ impl Token {
     }
 
     /// Is this token a standard symbol used in the language?
-    pub fn is_symbol(&self) -> bool {
+    #[must_use]
+    pub const fn is_symbol(&self) -> bool {
         use Token::*;
 
         match self {
@@ -905,7 +917,8 @@ impl Token {
     }
 
     /// Is this token an active standard keyword?
-    pub fn is_keyword(&self) -> bool {
+    #[must_use]
+    pub const fn is_keyword(&self) -> bool {
         use Token::*;
 
         match self {
@@ -924,7 +937,8 @@ impl Token {
 
     /// Is this token a reserved symbol?
     #[inline(always)]
-    pub fn is_reserved(&self) -> bool {
+    #[must_use]
+    pub const fn is_reserved(&self) -> bool {
         match self {
             Self::Reserved(_) => true,
             _ => false,
@@ -933,6 +947,7 @@ impl Token {
 
     /// Convert a token into a function name, if possible.
     #[cfg(not(feature = "no_function"))]
+    #[must_use]
     pub(crate) fn into_function_name_for_override(self) -> Result<String, Self> {
         match self {
             Self::Custom(s) | Self::Identifier(s) if is_valid_identifier(s.chars()) => Ok(s),
@@ -942,7 +957,8 @@ impl Token {
 
     /// Is this token a custom keyword?
     #[inline(always)]
-    pub fn is_custom(&self) -> bool {
+    #[must_use]
+    pub const fn is_custom(&self) -> bool {
         match self {
             Self::Custom(_) => true,
             _ => false,
@@ -991,6 +1007,7 @@ pub trait InputStream {
     /// Get the next character from the `InputStream`.
     fn get_next(&mut self) -> Option<char>;
     /// Peek the next character in the `InputStream`.
+    #[must_use]
     fn peek_next(&mut self) -> Option<char>;
 }
 
@@ -1028,6 +1045,7 @@ pub trait InputStream {
 /// # Volatile API
 ///
 /// This function is volatile and may change.
+#[must_use]
 pub fn parse_string_literal(
     stream: &mut impl InputStream,
     state: &mut TokenizeState,
@@ -1095,7 +1113,7 @@ pub fn parse_string_literal(
 
         match next_char {
             // \r - ignore if followed by \n
-            '\r' if stream.peek_next().map(|ch| ch == '\n').unwrap_or(false) => {}
+            '\r' if stream.peek_next().map(|ch| ch == '\n').unwrap_or(false) => (),
             // \...
             '\\' if !verbatim && escape.is_empty() => {
                 escape.push('\\');
@@ -1287,6 +1305,7 @@ fn scan_block_comment(
 ///
 /// This function is volatile and may change.
 #[inline(always)]
+#[must_use]
 pub fn get_next_token(
     stream: &mut impl InputStream,
     state: &mut TokenizeState,
@@ -1326,12 +1345,14 @@ fn is_numeric_digit(c: char) -> bool {
 #[cfg(not(feature = "no_function"))]
 #[cfg(feature = "metadata")]
 #[inline(always)]
+#[must_use]
 pub fn is_doc_comment(comment: &str) -> bool {
     (comment.starts_with("///") && !comment.starts_with("////"))
         || (comment.starts_with("/**") && !comment.starts_with("/***"))
 }
 
 /// Get the next token.
+#[must_use]
 fn get_next_token_inner(
     stream: &mut impl InputStream,
     state: &mut TokenizeState,
@@ -1408,7 +1429,7 @@ fn get_next_token_inner(
 
                 while let Some(next_char) = stream.peek_next() {
                     match next_char {
-                        ch if valid(ch) || ch == NUM_SEP => {
+                        ch if valid(ch) || ch == NUMBER_SEPARATOR => {
                             result.push(next_char);
                             eat_next(stream, pos);
                         }
@@ -1509,8 +1530,11 @@ fn get_next_token_inner(
                 // Parse number
                 return Some((
                     if let Some(radix) = radix_base {
-                        let out: String =
-                            result.iter().skip(2).filter(|&&c| c != NUM_SEP).collect();
+                        let out: String = result
+                            .iter()
+                            .skip(2)
+                            .filter(|&&c| c != NUMBER_SEPARATOR)
+                            .collect();
 
                         INT::from_str_radix(&out, radix)
                             .map(Token::IntegerConstant)
@@ -1518,7 +1542,8 @@ fn get_next_token_inner(
                                 Token::LexError(LERR::MalformedNumber(result.into_iter().collect()))
                             })
                     } else {
-                        let out: String = result.iter().filter(|&&c| c != NUM_SEP).collect();
+                        let out: String =
+                            result.iter().filter(|&&c| c != NUMBER_SEPARATOR).collect();
                         let num = INT::from_str(&out).map(Token::IntegerConstant);
 
                         // If integer parsing is unnecessary, try float instead
@@ -1962,6 +1987,7 @@ fn get_identifier(
 
 /// Is this keyword allowed as a function?
 #[inline(always)]
+#[must_use]
 pub fn is_keyword_function(name: &str) -> bool {
     match name {
         KEYWORD_PRINT | KEYWORD_DEBUG | KEYWORD_TYPE_OF | KEYWORD_EVAL | KEYWORD_FN_PTR
@@ -1975,6 +2001,7 @@ pub fn is_keyword_function(name: &str) -> bool {
 }
 
 /// Is a text string a valid identifier?
+#[must_use]
 pub fn is_valid_identifier(name: impl Iterator<Item = char>) -> bool {
     let mut first_alphabetic = false;
 
@@ -1994,6 +2021,7 @@ pub fn is_valid_identifier(name: impl Iterator<Item = char>) -> bool {
 /// Is a character valid to start an identifier?
 #[cfg(feature = "unicode-xid-ident")]
 #[inline(always)]
+#[must_use]
 pub fn is_id_first_alphabetic(x: char) -> bool {
     unicode_xid::UnicodeXID::is_xid_start(x)
 }
@@ -2001,6 +2029,7 @@ pub fn is_id_first_alphabetic(x: char) -> bool {
 /// Is a character valid for an identifier?
 #[cfg(feature = "unicode-xid-ident")]
 #[inline(always)]
+#[must_use]
 pub fn is_id_continue(x: char) -> bool {
     unicode_xid::UnicodeXID::is_xid_continue(x)
 }
@@ -2008,6 +2037,7 @@ pub fn is_id_continue(x: char) -> bool {
 /// Is a character valid to start an identifier?
 #[cfg(not(feature = "unicode-xid-ident"))]
 #[inline(always)]
+#[must_use]
 pub fn is_id_first_alphabetic(x: char) -> bool {
     x.is_ascii_alphabetic()
 }
@@ -2015,6 +2045,7 @@ pub fn is_id_first_alphabetic(x: char) -> bool {
 /// Is a character valid for an identifier?
 #[cfg(not(feature = "unicode-xid-ident"))]
 #[inline(always)]
+#[must_use]
 pub fn is_id_continue(x: char) -> bool {
     x.is_ascii_alphanumeric() || x == '_'
 }
@@ -2140,7 +2171,7 @@ impl<'a> Iterator for TokenIterator<'a> {
                 ("::<", false) => Token::LexError(LERR::ImproperSymbol(s,
                     "'::<>' is not a valid symbol. This is not Rust! Should it be '::'?".to_string(),
                 )),
-                ("(*", false) | ("*)", false) | ("begin", false) | ("end", false) => Token::LexError(LERR::ImproperSymbol(s,
+                ("(*", false) | ("*)", false) => Token::LexError(LERR::ImproperSymbol(s,
                     "'(* .. *)' is not a valid comment format. This is not Pascal! Should it be '/* .. */'?".to_string(),
                 )),
                 ("#", false) => Token::LexError(LERR::ImproperSymbol(s,
@@ -2201,6 +2232,7 @@ impl Engine {
     /// Exported under the `internals` feature only.
     #[cfg(feature = "internals")]
     #[inline(always)]
+    #[must_use]
     pub fn lex<'a>(
         &'a self,
         input: impl IntoIterator<Item = &'a &'a str>,
@@ -2211,6 +2243,7 @@ impl Engine {
     /// Exported under the `internals` feature only.
     #[cfg(feature = "internals")]
     #[inline(always)]
+    #[must_use]
     pub fn lex_with_map<'a>(
         &'a self,
         input: impl IntoIterator<Item = &'a &'a str>,
@@ -2220,6 +2253,7 @@ impl Engine {
     }
     /// Tokenize an input text stream with an optional mapping function.
     #[inline(always)]
+    #[must_use]
     pub(crate) fn lex_raw<'a>(
         &'a self,
         input: impl IntoIterator<Item = &'a &'a str>,
