@@ -208,13 +208,13 @@ fn test_arrays_map_reduce() -> Result<(), Box<EvalAltResult>> {
 
     assert_eq!(
         engine.eval::<INT>(
-            r#"
+            "
                 let x = [1, 2, 3];
                 x.reduce(|sum, v, i| {
                     if i == 0 { sum = 10 }
                     sum + v * v
                 })
-            "#
+            "
         )?,
         24
     );
@@ -231,40 +231,40 @@ fn test_arrays_map_reduce() -> Result<(), Box<EvalAltResult>> {
 
     assert_eq!(
         engine.eval::<INT>(
-            r#"
+            "
                 let x = [1, 2, 3];
                 x.reduce_rev(|sum, v, i| { if i == 2 { sum = 10 } sum + v * v })
-            "#
+            "
         )?,
         24
     );
 
     assert!(engine.eval::<bool>(
-        r#"
+        "
             let x = [1, 2, 3];
             x.some(|v| v > 1)
-        "#
+        "
     )?);
 
     assert!(engine.eval::<bool>(
-        r#"
+        "
             let x = [1, 2, 3];
             x.some(|v, i| v * i == 0)
-        "#
+        "
     )?);
 
     assert!(!engine.eval::<bool>(
-        r#"
+        "
             let x = [1, 2, 3];
             x.all(|v| v > 1)
-        "#
+        "
     )?);
 
     assert!(engine.eval::<bool>(
-        r#"
+        "
             let x = [1, 2, 3];
             x.all(|v, i| v > i)
-        "#
+        "
     )?);
 
     Ok(())
